# Generated by Django 4.2.1 on 2023-06-24 18:22

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Outpost',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identifier', models.CharField(max_length=400, unique=True, validators=[django.core.validators.RegexValidator(message='identifier doesnt match condition ^[A-Za-z0-9\\.\\-:_,=\\/@]+$', regex='^[A-Za-z0-9\\.\\-:_,=\\/@]+$'), django.core.validators.MaxLengthValidator(400), django.core.validators.MinLengthValidator(1)])),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('msg_queue_info', models.JSONField(default=dict)),
                ('msg', models.JSONField(default=dict)),
                ('status', models.CharField(choices=[('P', 'PENDING'), ('I', 'IN-PROGRESS'), ('E', 'ERROR'), ('C', 'COMPLETED')], default='P', max_length=1)),
                ('attempts', models.IntegerField(default=0)),
                ('debug_info', models.JSONField(default=dict)),
            ],
            options={
                'verbose_name': 'Outpost',
                'verbose_name_plural': 'Outpost',
            },
        ),
    ]



"""
Provides JSON rendering support.
"""
from __future__ import unicode_literals
import json

from rest_framework.renderers import BaseRenderer
from kfsd.apps.core.common.yaml import Yaml


class KubefacetsJSONRenderer(BaseRenderer):
    """
    Renderer which serializes to JSON.
    """

    media_type = "application/json"
    format = "json"
    charset = "utf-8"
    ensure_ascii = False
    default_flow_style = False

    def render(self, data, accepted_media_type=None, renderer_context=None):
        """
        Renders `data` into serialized JSON.
        """
        if data is None:
            return ""
        customYaml = Yaml()
        try:
            formattedData = customYaml.getPythonObj(customYaml.formatDictConfig(data))
            jsonIndented = json.dumps(formattedData, indent=4)
            return jsonIndented
        except Exception:
            return json.dumps(data, indent=4)



from pathlib import Path

BASE_DIR = Path(__file__).resolve().parent.parent

SECRET_KEY = "django-insecure-1=9rra#bge4g1rt$lxylq@%n*0ai@sl^qb%*ih(6i)9te24&ne"

DEBUG = True

ALLOWED_HOSTS = []

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "kfsd.apps.frontend",
    "kfsd.apps.core",
    "kfsd.apps.models",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "kfsd.apps.core.middleware.config.KubefacetsConfigMiddleware",
    "kfsd.apps.core.middleware.token.KubefacetsTokenMiddleware",
]

ROOT_URLCONF = "kfsd.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "kfsd.wsgi.application"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
    }
}

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True

STATIC_URL = "static/"

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

REST_FRAMEWORK = {
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
    "KUBEFACETS": {"STACKTRACE": False},
}

SPECTACULAR_SETTINGS = {
    "TITLE": "KFSD Utils as a Service",
    "VERSION": "1.0.0",
    "COMPONENT_SPLIT_REQUEST": True,
    "ENUM_ADD_EXPLICIT_BLANK_NULL_CHOICE": False,
    "SERVE_INCLUDE_SCHEMA": False,
    "POSTPROCESSING_HOOKS": [],
    "SERVE_AUTHENTICATION": None,
    "APPEND_COMPONENTS": {
        "securitySchemes": {
            "ApiKeyAuth": {"type": "apiKey", "in": "header", "name": "X-APIKey"}
        }
    },
    "SECURITY": [
        {
            "ApiKeyAuth": [],
        }
    ],
}

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "default-cache",
    }
}

KUBEFACETS = {
    "config": {
        "is_local_config": False,
        "lookup_dimension_keys": ["env"],
        "local": [
            {
                "setting": ["master"],
                "app": "app_api_utils_as_a_service",
                "services": {
                    "api_key": "9a02f7923aa22e69e0e2858d682a0c227ae0f3ce125a41c61d",
                    "gateway": {
                        "host": "http://localhost:8002/apis",
                        "sso": {"verify_tokens_uri": "sso/cookies/verify/"},
                        "core": {"common_config_uri": "core/config/common/"},
                    },
                },
            },
            {"setting": ["env:dev"], "certs": {"host": "http://localhost:8002/apis"}},
        ],
    },
}



from kfsd.apps.core.auth.token import TokenUser
from kfsd.apps.core.utils.http.django.response import DjangoResponse


class KubefacetsTokenMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request, *args, **kwargs):
        request.token_user = TokenUser(request)
        response = self.get_response(request)
        return self.processResponse(response, request)

    def processResponse(self, response, request):
        cookies = request.token_user.getUserCookies()
        djangoResponse = DjangoResponse(response)
        if cookies:
            for cookie in cookies:
                djangoResponse.setCookie(**cookie)
        return djangoResponse.getResponse()



from kfsd.apps.endpoints.serializers.base import ErrorSerializer
from kfsd.apps.endpoints.views.docs.v1.common import CommonV1Doc
from kfsd.apps.endpoints.serializers.common.configuration import ConfigurationInputReqSerializer, ConfigurationOutputRespSerializer


class CommonDoc:
    @staticmethod
    def config_view():
        return {
            "summary": "Config",
            "tags": ["COMMON"],
            "request": ConfigurationInputReqSerializer,
            "responses": {
                200: ConfigurationOutputRespSerializer,
                500: ErrorSerializer
            },
            "examples": CommonV1Doc.config_examples()
        }



from kfsd.apps.core.tests.base_api import BaseAPITestCases
from kfsd.apps.core.utils.system import System
from kfsd.apps.core.utils.file import FileUtils


class SystemTests(BaseAPITestCases):

    def setUp(self):
        self.__systemObj = System()
        return super().setUp()

    def test_checksum(self):
        checksumStr = "My name is Nathan"
        checksum = System.checksum(checksumStr)
        self.assertEqual(checksum, 878446078)

        checksumDict = {"name": "Nathan"}
        checksum1 = System.checksum(checksumDict)
        self.assertEqual(checksum1, 915277270)

        checksumList = ["name", "nathan"]
        checksum2 = System.checksum(checksumList)
        self.assertEqual(checksum2, 882116008)

        checksumInt = 123
        checksum3 = System.checksum(checksumInt)
        self.assertEqual(checksum3, 8060929)

    def test_uuid(self):
        length = 10
        uuid = System.uuid(length)
        self.assertTrue(len(uuid) == length)

    def test_secret(self):
        length = 10
        secretCode = System.secret(length)
        self.assertTrue(len(secretCode) > length)

    def test_api_key(self):
        length = 10
        apiKey = System.api_key(length)
        self.assertTrue(len(apiKey) == length)

    def test_encrypt_key(self):
        encryptKey = self.__systemObj.encryptKey()
        self.assertTrue(self.__systemObj.getCmdExecStatus())
        self.assertTrue(len(encryptKey) > 40)

    def test_os_arch(self):
        arch = self.__systemObj.osArch()
        self.assertTrue(len(arch) > 2)

    def test_exec_cmd(self):
        cmd = "pwd"
        self.__systemObj.cmdExec(cmd, False)
        self.assertIsNone(self.__systemObj.getCmdExecOutput())

    def test_host_ip(self):
        hostIP = self.__systemObj.hostIP()
        self.assertTrue(len(hostIP) > 6)

    def test_hostname(self):
        self.assertIsNotNone(self.__systemObj.getHostName())

    def test_get_os(self):
        self.assertIsNotNone(self.__systemObj.getOS())

    def test_change_file_permission(self):
        filePath = "/tmp/utils/tmp.txt"
        writeContent = "my content"
        FileUtils.write(filePath, writeContent)

        permission = self.__systemObj.changeFilePermission("777", filePath)
        self.assertTrue(permission)



import pika
from kfsd.apps.core.common.logger import Logger, LogLevel
from kfsd.apps.core.common.kubefacets_config import KubefacetsConfig
from kfsd.apps.core.utils.http.django.config import DjangoConfig
from kfsd.apps.core.utils.dict import DictUtils

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class RabbitMQ:
    def __init__(self):
        self.__config = DjangoConfig(KubefacetsConfig().getConfig())
        self.__connection = self.connect()
        self.__channel = self.__connection.channel()
        self.__queueName = ""
        self.__exchangeName = ""
        self.__routingKey = ""
        self.__exchangeType = "topic"
        self.__isQueueExclusive = False
        self.__queue = None
        self.__exchange = None
        self.__autoAck = True
        self.__queueDurable = True
        self.__exchangeDurable = True

    def getConfig(self):
        return self.__config

    def connect(self):
        connectionConfig = self.__config.findConfigs(["services.rabbitmq.connect"])[
            0
        ].copy()
        authCredentials = connectionConfig.pop("credentials")
        connectionConfig["credentials"] = self.constructCredentials(
            DictUtils.get(authCredentials, "username"),
            DictUtils.get(authCredentials, "pwd"),
        )
        connection_params = pika.ConnectionParameters(**connectionConfig)
        return pika.BlockingConnection(connection_params)

    def constructCredentials(self, username, pwd):
        return pika.PlainCredentials(username, pwd)

    def setQueueName(self, queueName):
        self.__queueName = queueName

    def setAutoAck(self, ack):
        self.__autoAck = ack

    def setQueueExclusive(self, exclusiveVal):
        self.__isQueueExclusive = exclusiveVal

    def setExchangeType(self, exchangeType):
        self.__exchangeType = exchangeType

    def setExchangeName(self, exchangeName):
        self.__exchangeName = exchangeName

    def setRoutingKey(self, routingKey):
        self.__routingKey = routingKey

    def declareQueue(self):
        self.__queue = self.__channel.queue_declare(
            queue=self.__queueName,
            exclusive=self.__isQueueExclusive,
            durable=self.__queueDurable,
        )

    def declareExchange(self):
        self.__exchange = self.__channel.exchange_declare(
            exchange=self.__exchangeName,
            exchange_type=self.__exchangeType,
            durable=self.__exchangeDurable,
        )

    def queueBind(self):
        self.__channel.queue_bind(
            exchange=self.__exchangeName,
            queue=self.__queueName,
            routing_key=self.__routingKey,
        )

    def publish(self, msg):
        self.__channel.basic_publish(
            exchange=self.__exchangeName,
            routing_key=self.__routingKey,
            body=msg,
            properties=pika.BasicProperties(delivery_mode=2),
        )

    def publish_msg_and_close_connection(
        self, exchangeName, queueName, routingKey, msg
    ):
        self.publish_msg(exchangeName, queueName, routingKey, msg)
        self.closeConnection()

    def publish_msg(self, exchangeName, queueName, routingKey, msg):
        self.setExchangeName(exchangeName)
        self.setQueueName(queueName)
        self.setRoutingKey(routingKey)
        self.declareExchange()
        self.declareQueue()
        self.publish(msg)

    def consume_msgs(self, callback, exchangeName, queueName, routingKey):
        self.setExchangeName(exchangeName)
        self.setQueueName(queueName)
        self.setRoutingKey(routingKey)
        self.declareExchange()
        self.declareQueue()
        self.queueBind()
        self.consume(callback)
        self.startConsuming()

    def consume(self, callback):
        self.__channel.basic_consume(
            queue=self.__queueName,
            on_message_callback=callback,
            auto_ack=self.__autoAck,
        )

    def closeConnection(self):
        self.__connection.close()

    def startConsuming(self):
        self.__channel.start_consuming()



from django.core.management.base import BaseCommand
from kfsd.apps.core.msmq.rabbitmq.base import RabbitMQ
from kfsd.apps.core.utils.time import Time
from kfsd.apps.endpoints.serializers.common.outpost import (
    MsgSerializer,
    OutpostModelSerializer,
)
from kfsd.apps.core.common.logger import Logger, LogLevel
from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.models.tables.outpost import Outpost, send_msg
from kfsd.apps.models.constants import (
    SERVICE_CONFIG_ID_REMIND,
    COMMON_ACTION_OUTPOST_CLEAR,
    REMIND_ACTION_CREATE,
    REMIND_ACTION_IN_MINS,
)

import json

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


def send_clear_outpost_reminder(config, serviceConfigId):
    identifier = "ORG=Kubefacets,APP={},TYPE=Outpost,Action=Clear".format(
        serviceConfigId
    )
    clientExchangeName, clientQueueName, clientRoutingKey = get_service_msmq_details(
        config, serviceConfigId
    )
    remindExchangeName, remindQueueName, remindRoutingKey = get_service_msmq_details(
        config, SERVICE_CONFIG_ID_REMIND
    )
    outpostData = {
        "msg_queue_info": {
            "exchange_name": remindExchangeName,
            "queue_name": remindQueueName,
            "routing_key": remindRoutingKey,
        },
        "msg": {
            "action": REMIND_ACTION_CREATE,
            "data": {
                "identifier": identifier,
                "type": "REPEAT",
                "remind_by_in_mins": REMIND_ACTION_IN_MINS,
                "to_msg_queue": {
                    "exchange_name": clientExchangeName,
                    "queue_name": clientQueueName,
                    "routing_key": clientRoutingKey,
                },
                "msg": {
                    "action": COMMON_ACTION_OUTPOST_CLEAR,
                    "data": {"identifier": identifier},
                },
            },
        },
    }
    outpostSerializer = OutpostModelSerializer(data=outpostData)
    outpostSerializer.is_valid(raise_exception=True)
    outpostSerializer.save()


def get_service_msmq_details(config, serviceConfigId):
    return config.findConfigs(
        [
            "services.rabbitmq.consume.{}.exchange_name".format(serviceConfigId),
            "services.rabbitmq.consume.{}.queue_name".format(serviceConfigId),
            "services.rabbitmq.consume.{}.routing_key".format(serviceConfigId),
        ]
    )


def clear_outpost(data):
    outpostQS = Outpost.objects.filter(status="E")
    identifiers = [outpost.identifier for outpost in outpostQS]
    logger.info(
        "Recd CLEAR_OUTPOST command on msmq, identifiers: {} with 'Error' status.".format(
            identifiers
        )
    )
    for outpostIns in outpostQS:
        send_msg(outpostIns.id)


callback_map = {COMMON_ACTION_OUTPOST_CLEAR: clear_outpost}


def base_callback(ch, method, properties, body):
    bodyStr = body.decode().replace("'", '"')
    jsonStr = json.loads(bodyStr)
    serializedData = MsgSerializer(data=jsonStr)
    serializedData.is_valid(raise_exception=True)

    action = DictUtils.get(serializedData.data, "action")
    callback_map[action](serializedData.data)


class Command(BaseCommand):
    help = "Listens to a RabbitMQ topic"

    def __init__(self, callbackFn=base_callback):
        self.__callbackFn = callbackFn

    def add_arguments(self, parser):
        parser.add_argument(
            "-s",
            "--service_config_id",
            type=str,
            help="Service Config Id",
        )

    def connectToMSMQ(self):
        try:
            msmqHandler = RabbitMQ()
            return msmqHandler
        except Exception:
            logger.error(
                "Error connecting to RabbitMQ, check if RabbitMQ instance is up!"
            )
            Time.sleep(30)
            self.connectToMSMQ()

    def consumeMsgs(self, serviceConfigId, config, msmqHandler):
        exchangeName, queueName, routingKey = get_service_msmq_details(
            config, serviceConfigId
        )
        logger.info(
            "Listening msgs at exchange: {}, queue: {}, routing key: {}".format(
                exchangeName, queueName, routingKey
            )
        )
        msmqHandler.consume_msgs(self.__callbackFn, exchangeName, queueName, routingKey)

    def handle(self, *args, **options):
        logger.info("Listening to MSMQ messages...")
        serviceConfigId = DictUtils.get(options, "service_config_id")
        msmqHandler = self.connectToMSMQ()
        config = msmqHandler.getConfig()
        send_clear_outpost_reminder(config, serviceConfigId)
        self.consumeMsgs(serviceConfigId, config, msmqHandler)



from kfsd.apps.core.tests.base_api import BaseAPITestCases
from kfsd.apps.core.common.logger import Logger, LogLevel

import io


class LoggerTests(BaseAPITestCases):

    def test_logger_info(self):
        stream = io.StringIO()
        logger = Logger.getSingleton(__name__, LogLevel.INFO, stream)

        # info
        logger.info('This is a info message')
        stream_contents = stream.getvalue()
        self.assertIn('This is a info message', stream_contents)

        # warning
        logger.warn('This is a warning message')
        stream_contents = stream.getvalue()
        self.assertIn('This is a warning message', stream_contents)

        # error
        logger.error('This is a error message')
        stream_contents = stream.getvalue()
        self.assertIn('This is a error message', stream_contents)

        # critical
        logger.critical('This is a critical message')
        stream_contents = stream.getvalue()
        self.assertIn('This is a critical message', stream_contents)

        # debug
        logger.debug('This is a debug message')
        stream_contents = stream.getvalue()
        self.assertNotIn('This is a debug message', stream_contents)

    def test_logger_debug(self):
        stream = io.StringIO()
        logger = Logger.getSingleton(__name__, LogLevel.DEBUG, stream)

        logger.debug('This is a debug message')
        stream_contents = stream.getvalue()
        self.assertIn('This is a debug message', stream_contents)



from rest_framework import status
from kfsd.apps.core.auth.api.gateway import APIGateway


class SSO(APIGateway):
    def __init__(self, request=None):
        APIGateway.__init__(self, request)

    def getVerifyTokensUrl(self):
        return self.constructUrl(
            ["services.gateway.host", "services.gateway.sso.verify_tokens_uri"]
        )

    def verifyTokens(self, payload):
        return self.httpPost(self.getVerifyTokensUrl(), payload, status.HTTP_200_OK)



from functools import wraps


class ExecException(Exception):
    def __init__(self, logger, msg):
        logger.exception(msg)


def ExecExceptionHandler(logger):
    def exception(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                raise ExecException(logger, e.__str__())
        return wrapper
    return exception



import http.cookies
from kfsd.apps.core.utils.dict import DictUtils


class Cookie:
    COOKIE_HEADER_KEY = "Cookie"

    def __init__(self, cookies={}):
        self.__httpObj = None
        self.__cookies = cookies

    def getCookiesHttpObj(self):
        return self.__httpObj

    def setCookiesHttpObj(self, httpObj):
        self.__httpObj = httpObj

    def getCookies(self):
        return self.__cookies

    def setCookies(self, cookies, collection=True):
        if not collection:
            self.__cookies = DictUtils.merge(dict1=self.__cookies, dict2=cookies)
        else:
            self.__cookies = cookies

    def hasCookies(self):
        if self.getCookies():
            return True
        return False

    def cookiesToHeaderStr(self, rmCookieKeys=[]):
        cookie_string = ""
        if self.__cookies:
            cookie = http.cookies.SimpleCookie()
            filteredCookies = DictUtils.filter_by_keys_neg(self.__cookies, rmCookieKeys)
            cookie.update(filteredCookies)
            for key, value in cookie.items():
                cookie_string += f"{key}={value}; "
            # remove the trailing '; '
            cookie_string = cookie_string[:-2]
        return cookie_string



MIN_LENGTH = 1
MAX_LENGTH = 400

IDENTIFIER_REGEX_CONDITION = r"^[A-Za-z0-9\.\-:_,=\/@]+$"
MSMQ_NAME_REGEX_CONDITION = r"^[A-Za-z0-9_\.]+$"


ORG = "ORG"
APPLICATION = "APP"
PROJECT = "PRJ"
TYPE = "TYPE"
NAMESPACE = "NS"

# SERVICE_CONFIG_ID
SERVICE_CONFIG_ID_REMIND = "remind"
SERVICE_CONFIG_ID_UTILS = "utils"
SERVICE_CONFIG_ID_SSO_FE = "sso_fe"
SERVICE_CONFIG_ID_SSO_API = "sso_api"
SERVICE_CONFIG_ID_GATEWAY = "gateway"
SERVICE_CONFIG_ID_CERTS = "certs"
SERVICE_CONFIG_ID_PERMS = "perms"

# COMMON ACTIONS
COMMON_ACTION_OUTPOST_CLEAR = "CLEAR_OUTPOST"
REMIND_ACTION_CREATE = "CREATE_REMINDER"
REMIND_ACTION_DELETE = "DELETE_REMINDER"
REMIND_ACTION_IN_MINS = 3



from drf_spectacular.utils import extend_schema
from rest_framework import viewsets, decorators, renderers, response, status

from kfsd.apps.endpoints.views.docs.utils import UtilsDoc
from kfsd.apps.endpoints.handlers.utils.arr import ArrHandler
from kfsd.apps.endpoints.handlers.utils.attr import AttrHandler
from kfsd.apps.endpoints.handlers.utils.system import SystemHandler
from kfsd.apps.endpoints.serializers.utils.arr import ArrUtilsInputReqSerializer
from kfsd.apps.endpoints.serializers.utils.attr import AttrUtilsInputReqSerializer
from kfsd.apps.endpoints.serializers.utils.system import SystemInputReqSerializer


class UtilsViewSet(viewsets.ViewSet):
    lookup_field = "identifier"
    lookup_value_regex = '[^/]+'

    def parseInput(self, request, serializer):
        inputSerializer = serializer(data=request.data)
        inputSerializer.is_valid()
        return inputSerializer.data

    def getSystemInputData(self, request):
        return self.parseInput(request, SystemInputReqSerializer)

    def getArrInputData(self, request):
        return self.parseInput(request, ArrUtilsInputReqSerializer)

    def getAttrInputData(self, request):
        return self.parseInput(request, AttrUtilsInputReqSerializer)

    @extend_schema(**UtilsDoc.system_view())
    @decorators.action(detail=False, methods=['post'], renderer_classes=[renderers.JSONRenderer])
    def system(self, request):
        systemHandler = SystemHandler(self.getSystemInputData(request))
        return response.Response(systemHandler.gen(), status.HTTP_200_OK)

    @extend_schema(**UtilsDoc.arr_view())
    @decorators.action(detail=False, methods=['post'], renderer_classes=[renderers.JSONRenderer])
    def arr(self, request):
        arrHandler = ArrHandler(self.getArrInputData(request))
        return response.Response(arrHandler.gen(), status.HTTP_200_OK)

    @extend_schema(**UtilsDoc.attr_view())
    @decorators.action(detail=False, methods=['post'], renderer_classes=[renderers.JSONRenderer])
    def attr(self, request):
        attrHandler = AttrHandler(self.getAttrInputData(request))
        return response.Response(attrHandler.gen(), status.HTTP_200_OK)



from enum import Enum
from functools import wraps
from kfsd.apps.core.common.singleton import Singleton

import logging
import json


class Logger:
    __defaultFormat = "[%(asctime)s][%(name)s][%(levelname)s] %(message)s"

    def __init__(self, fileName, logLevel, stream=None):
        self.__logger = logging.getLogger(fileName)
        self.__handler = self.setStreamHandler(stream)
        self.__getLoggerLevel(logLevel)
        self.setFormat(self.__defaultFormat)

    @classmethod
    @Singleton
    def getSingleton(cls, fileName, logLevel, stream=None):
        return cls(fileName, logLevel, stream)

    def setStreamHandler(self, stream):
        return logging.StreamHandler(stream)

    def getLogger(self):
        return self.__logger

    def getHandler(self):
        return self.__handler

    def setFormat(self, format):
        formatter = logging.Formatter(format)
        self.__handler.setFormatter(formatter)
        self.__logger.addHandler(self.__handler)

    def __getLogger(self):
        return self.__logger

    def debug(self, msg, *args, **kwargs):
        self.__getLogger().debug(msg, *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        self.__getLogger().info(msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        self.__getLogger().error(msg, *args, **kwargs)

    def warn(self, msg, *args, **kwargs):
        self.__getLogger().warn(msg, *args, **kwargs)

    def exception(self, msg, *args, **kwargs):
        self.__getLogger().exception(msg, *args, **kwargs)

    def critical(self, msg, *args, **kwargs):
        self.__getLogger().critical(msg, *args, **kwargs)

    def __getLoggerLevel(self, logLevel):
        if logLevel is not None:
            loggerLevelDict = {}
            loggerLevelDict[str(LogLevel.CRITICAL)] = logging.CRITICAL
            loggerLevelDict[str(LogLevel.ERROR)] = logging.ERROR
            loggerLevelDict[str(LogLevel.WARNING)] = logging.WARNING
            loggerLevelDict[str(LogLevel.INFO)] = logging.INFO
            loggerLevelDict[str(LogLevel.DEBUG)] = logging.DEBUG
            loggerLevelDict[str(LogLevel.NOTSET)] = logging.NOTSET
            loggingLevel = (
                loggerLevelDict[str(logLevel)]
                if str(logLevel) in loggerLevelDict
                else logging.NOTSET
            )
            return self.__logger.setLevel(loggingLevel)

        return self.__logger.setLevel(logging.NOTSET)

    def getLogMethod(self, errorType):
        logMap = {
            "error": self.error,
            "warn": self.warn,
            "debug": self.debug,
            "info": self.info,
            "critical": self.critical,
        }
        if errorType in logMap:
            return logMap[errorType]
        else:
            return self.error

    def getAttr(self, obj, attr):
        if hasattr(obj, attr):
            return getattr(obj, attr)
        return None

    def logWebRequestError(self, request, error, errorType):
        x_apikey = request.headers["X_Apikey"] if "X_Apikey" in request.headers else ""
        errorData = {
            "request_info": {
                "path": request.build_absolute_uri(),
                "method": self.getAttr(request, "method"),
                "api_key": x_apikey,
                "content_type": self.getAttr(request, "content_type"),
                "query_params": self.getAttr(request, "query_params"),
                "body": self.getAttr(request, "data")
            },
            "error": error,
        }
        errorStr = json.dumps(errorData, indent=4)

        self.getLogMethod(errorType)(errorStr)


class LogLevel(Enum):
    CRITICAL = 0
    ERROR = 1
    WARNING = 2
    INFO = 3
    DEBUG = 4
    NOTSET = 5


def InstanceDebug(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        selfObj = args[0]
        result = func(*args, **kwargs)
        selfObj.getLogObj().debug("Input:{}, Output: {}".format(args, result))
        return result

    return wrapper


def Debug(logger):
    def debugWrapper(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)
            logger.debug("Input:{}, Output: {}".format(args, result))
            return result

        return wrapper

    return debugWrapper



from rest_framework.test import APITestCase
from django.urls import reverse
from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.utils.file import FileUtils


class EndpointsTestHandler(APITestCase):

    def setUp(self):
        return super().setUp()

    def tearDown(self):
        return super().tearDown()

    def readJSONData(self, filepath):
        return FileUtils.read_as_json(filepath)

    def fetchPg(self, listUrl, currentPg):
        if currentPg:
            listUrl = listUrl + "?page={}".format(currentPg)
        return listUrl

    def detailView(self, name, identifier):
        return reverse(name, args=[identifier])

    def createView(self, name):
        return reverse(name)

    def rmAttrs(self, resp, stripAttrs=["created", "updated", "id"]):
        if isinstance(resp, dict):
            return DictUtils.filter_by_keys_neg(resp, stripAttrs)
        return resp

    def rmAttrsFromResults(self, resp, stripAttrs=["created", "updated", "id"]):
        formattedResults = []
        results = DictUtils.get(resp, "results")
        if results:
            for item in results:
                strippedItem = self.rmAttrs(item, stripAttrs)
                formattedResults.append(strippedItem)

        resp["results"] = formattedResults
        return resp

    def list(self, url, currentPg, expStatus, stripAttrs=["created", "updated", "id"]):
        paginatedUrl = self.fetchPg(url, currentPg)
        response = self.client.get(paginatedUrl, format='json')
        self.assertEqual(response.status_code, expStatus, response.data)
        return self.rmAttrsFromResults(response.data, stripAttrs) if stripAttrs else response.data

    def get(self, name, identifier, expStatus, stripAttrs=["created", "updated", "id"]):
        detailUrl = self.detailView(name, identifier)
        response = self.client.get(detailUrl, format='json')
        self.assertEqual(response.status_code, expStatus, response.data)
        return self.rmAttrs(response.data, stripAttrs) if stripAttrs else response.data

    def create(self, url, data, expStatus, stripAttrs=["created", "updated", "id"]):
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, expStatus, response.data)
        return self.rmAttrs(response.data, stripAttrs) if stripAttrs else response.data

    def post(self, url, data, expStatus, stripAttrs=["created", "updated", "id"]):
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, expStatus, response.data)
        return self.rmAttrs(response.data, stripAttrs) if stripAttrs else response.data

    def patch(self, name, identifier, data, expStatus, stripAttrs=["created", "updated", "id"]):
        detailUrl = self.detailView(name, identifier)
        response = self.client.patch(detailUrl, data=data, format='json')
        self.assertEqual(response.status_code, expStatus, response.data)
        return self.rmAttrs(response.data, stripAttrs) if stripAttrs else response.data

    def delete(self, name, identifier, expStatus):
        detailUrl = self.detailView(name, identifier)
        response = self.client.delete(detailUrl, format='json')
        self.assertEqual(response.status_code, expStatus, response.data)
        return response



import http.cookies

from kfsd.apps.core.utils.dict import DictUtils


class Cookie:
    KEY = 'key'
    COOKIE = 'cookie'
    EXPIRY_IN_SECS = 'expiry_in_secs'
    SECURE = 'secure'
    HTTP_ONLY = 'http_only'
    SAME_SITE = 'same_site'

    def __init__(self, httpObj):
        self.__httpObj = httpObj

    def getHttpObj(self):
        return self.__httpObj

    def setHttpObj(self, httpObj):
        self.__httpObj = httpObj

    def getAllCookies(self):
        return self.__httpObj.COOKIES

    def getCookie(self, key):
        return self.__httpObj.COOKIES.get(key)

    def setCookie(self, **kwargs):
        self.getHttpObj().set_cookie(
                key=kwargs[self.KEY],
                value=kwargs[self.COOKIE],
                expires=kwargs[self.EXPIRY_IN_SECS],
                secure=kwargs[self.SECURE],
                httponly=kwargs[self.HTTP_ONLY],
                samesite=kwargs[self.SAME_SITE]
            )

    def cookiesToHeaderStr(self, rmCookieKeys=[]):
        cookie_string = ''
        if self.getAllCookies():
            cookie = http.cookies.SimpleCookie()
            filteredCookies = DictUtils.filter_by_keys_neg(self.getAllCookies(), rmCookieKeys)
            cookie.update(filteredCookies)
            for key, value in cookie.items():
                cookie_string += f'{key}={value}; '
            # remove the trailing '; '
            cookie_string = cookie_string[:-2]
        return cookie_string



from kfsd.apps.core.tests.base_api import BaseAPITestCases
from kfsd.apps.core.utils.arr import ArrUtils


class ArrUtilsTests(BaseAPITestCases):

    def test_intersection(self):
        arr1 = [1, 2, 3, 7, 7]
        arr2 = [4, 7, 8, 2, 7]

        self.assertEqual(ArrUtils.intersection(arr1, arr2), [2, 7])

        self.assertEqual(ArrUtils.intersection([0, 6], [7, 8]), [])

    def test_merge(self):
        arr1 = [1, 2, 3, 7, 7]
        arr2 = [4, 7, 8, 2, 7]
        mergedArr = ArrUtils.merge(arr1, arr2, False)

        self.assertEqual(mergedArr, [1, 2, 2, 3, 4, 7, 7, 7, 7, 8])

        mergedArrUniq = ArrUtils.merge(arr1, arr2, True)

        self.assertEqual(mergedArrUniq, [1, 2, 3, 4, 7, 8])

    def test_join(self):
        arr1 = [1, 2, 3, 7, 7]
        joinedStr = ArrUtils.join(arr1, "|")

        self.assertEqual(joinedStr, "1|2|3|7|7")

    def test_sort_keys_by_len(self):
        keys = [
            ["env:k8s", "cluster:k8s", "type:dev"],
            ["env:k8s"],
            ["master"],
            ["cluster:inhouse", "type:dev"]
        ]

        sortedKeys = ArrUtils.sort_keys_by_len(keys)

        expKeys = [['env:k8s'], ['master'], ['cluster:inhouse', 'type:dev'], ['env:k8s', 'cluster:k8s', 'type:dev']]
        self.assertEqual(sortedKeys, expKeys)

        sortedKeysRev = ArrUtils.sort_keys_by_len(keys, True)
        expKeysRev = [['env:k8s', 'cluster:k8s', 'type:dev'], ['cluster:inhouse', 'type:dev'], ['env:k8s'], ['master']]
        self.assertEqual(sortedKeysRev, expKeysRev)

    def test_to_dict(self):
        arr1 = [
            {
                "id": 1,
                "name": "Gokul",
                "Gender": "M"
            },
            {
                "id": 2,
                "name": "Lavanya",
                "Gender": "F"
            }
        ]

        expDict = {
            1: {
                "id": 1,
                "name": "Gokul",
                "Gender": "M"
            },
            2: {
                "id": 2,
                "name": "Lavanya",
                "Gender": "F"
            }
        }

        obsDict = ArrUtils.to_dict(arr1, "id")
        self.assertEqual(obsDict, expDict)



from kfsd.apps.core.utils.dict import DictUtils

from rest_framework.exceptions import ValidationError
from django.db.models import Q
from django.shortcuts import get_object_or_404


class BaseHandler:
    ARG_NAME_IDENTIFIER = "identifier"
    ARG_NAME_SERIALIZER = "serializer"
    ARG_NAME_MODELCLASS = "modelClass"
    ARG_NAME_ISDBFETCH = "isDBFetch"

    def __init__(self, **kwargs):
        self.__modelSerializer = DictUtils.get(kwargs, self.ARG_NAME_SERIALIZER)
        self.__modelClass = DictUtils.get(kwargs, self.ARG_NAME_MODELCLASS)
        self.__identifier = DictUtils.get(kwargs, self.ARG_NAME_IDENTIFIER)
        isDBFetch = DictUtils.get(kwargs, self.ARG_NAME_ISDBFETCH)
        self.__modelQS = self.getQS(self.__identifier) if isDBFetch else None
        self.__modelQSData = self.getModelQSDataFromDB(self.__identifier) if isDBFetch else None

    def dbFetch(self):
        self.__modelQS = self.getQS(self.__identifier)
        self.__modelQSData = self.getModelQSDataFromDB(self.__identifier)

    def getModelClass(self):
        return self.__modelClass

    def getQS(self, objIdentifier):
        return get_object_or_404(self.getModelClass(), identifier=objIdentifier)

    def exists(self):
        if self.getModelClass().objects.filter(identifier=self.__identifier).exists():
            return True
        return False

    def create(self, **kwargs):
        return self.getModelClass().objects.create(**kwargs)

    def update(self, kwargs):
        serializer = self.getModelSerializer()(self.getModelQS(), data=kwargs, partial=True)
        if serializer.is_valid():
            return serializer.save()
        raise ValidationError(serializer.errors, "bad_request")

    def delete(self):
        if self.exists():
            self.getModelQS().delete()

    def getModelQSData(self):
        return self.__modelQSData

    def setModelQSData(self, data):
        self.__modelQSData = data

    def getModelQS(self, refresh=False):
        if refresh:
            self.__modelQS = self.getQS(self.__identifier)
        return self.__modelQS

    def refreshModelQSData(self):
        self.__modelQSData = self.getModelQSDataFromDB(self.__identifier)

    def getModelSerializer(self):
        return self.__modelSerializer

    def getIdentifier(self):
        return self.__identifier

    def getModelQSDataFromDB(self, objIdentifier):
        serializedData = self.getModelSerializer()(self.getQS(objIdentifier))
        return serializedData.data

    def getQStoSerializerMany(self, qs):
        serializedData = self.getModelSerializer()(qs, many=True)
        return serializedData

    def getIdentifiersQS(self, identifiers):
        query = Q(identifier__in=identifiers)
        return self.getModelClass().objects.distinct().filter(query)

    def getFilterQS(self, queries):
        return self.getModelClass().objects.distinct().filter(queries)

    def search(self, modelClass, queries):
        return modelClass.objects.distinct().filter(queries)



"""General API Views"""
from typing import Any
from django.urls import reverse
from django.views.generic import TemplateView
from kfsd.apps.frontend.views.permissions.base import PermissionView
from kfsd.apps.frontend.permissions.signin import SignInRequired
from kfsd.apps.frontend.permissions.staff import IsStaff
from kfsd.apps.frontend.permissions.active import IsActive
from kfsd.apps.frontend.permissions.email import SignUpEmailVerified


class APIDocsView(PermissionView, TemplateView):
    """Show browser view based on rapi-doc"""

    permission_classes = [SignInRequired, IsActive, SignUpEmailVerified, IsStaff]
    permission_classes_neg = []

    template_name = "v1/docs/browser.html"

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        path = self.request.build_absolute_uri(
            reverse(
                "schema-api",
            )
        )
        return super().get_context_data(path=path, **kwargs)



from rest_framework import status
from django.urls import reverse
from unittest.mock import patch

from kfsd.apps.endpoints.tests.endpoints_test_handler import EndpointsTestHandler
from kfsd.apps.core.utils.dict import DictUtils


class SystemUtilsTestCases(EndpointsTestHandler):
    def setUp(self):
        super().setUp()

    def get(self, url, expStatus, stripCommonAttrs=True):
        response = self.client.get(url, format='json')
        self.assertEqual(response.status_code, expStatus, response.data)
        return self.stripCommonAttrs(response.data) if stripCommonAttrs else response.data

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch("kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getConfig")
    def test_utils_system_checksum_str(self, genCommonConfigMocked, tokenUserInfoMocked):
        genCommonConfigMocked.side_effect = [{}]
        tokenUserInfoMocked.side_effect = [{}]
        postData = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/requests/utils/system/test_utils_system_checksum_str.json')
        obsResponse = self.post(reverse("utils-system"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/responses/utils/system/test_utils_system_checksum_str.json')
        self.assertEqual(obsResponse, expResponse)

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch("kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getConfig")
    def test_utils_system_checksum_dict(self, genCommonConfigMocked, tokenUserInfoMocked):
        genCommonConfigMocked.side_effect = [{}]
        tokenUserInfoMocked.side_effect = [{}]
        postData = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/requests/utils/system/test_utils_system_checksum_dict.json')
        obsResponse = self.post(reverse("utils-system"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/responses/utils/system/test_utils_system_checksum_dict.json')
        self.assertEqual(obsResponse, expResponse)

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch("kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getConfig")
    def test_utils_system_uuid(self, genCommonConfigMocked, tokenUserInfoMocked):
        genCommonConfigMocked.side_effect = [{}]
        tokenUserInfoMocked.side_effect = [{}]
        postData = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/requests/utils/system/test_utils_system_uuid.json')
        obsResponse = self.post(reverse("utils-system"), postData, status.HTTP_200_OK)
        uuid = DictUtils.get_by_path(obsResponse, "output.value")
        self.assertTrue(len(uuid) == 32)

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch("kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getConfig")
    def test_utils_system_secret(self, genCommonConfigMocked, tokenUserInfoMocked):
        genCommonConfigMocked.side_effect = [{}]
        tokenUserInfoMocked.side_effect = [{}]
        postData = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/requests/utils/system/test_utils_system_secret.json')
        obsResponse = self.post(reverse("utils-system"), postData, status.HTTP_200_OK)
        secret = DictUtils.get_by_path(obsResponse, "output.value")
        self.assertTrue(len(secret) > 32)

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch("kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getConfig")
    def test_utils_system_key(self, genCommonConfigMocked, tokenUserInfoMocked):
        genCommonConfigMocked.side_effect = [{}]
        tokenUserInfoMocked.side_effect = [{}]
        postData = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/requests/utils/system/test_utils_system_key.json')
        obsResponse = self.post(reverse("utils-system"), postData, status.HTTP_200_OK)
        key = DictUtils.get_by_path(obsResponse, "output.value")
        self.assertTrue(len(key) == 10)



from rest_framework.exceptions import APIException
from rest_framework import status
import json
import requests

from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.common.logger import Logger, LogLevel

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class KubefacetsAPIException(APIException):
    default_detail = "Unexpected Error"
    status_code = 500
    default_code = "unexpected_error"
    detailed_msg = ""

    def __init__(self, detail, defaultCode, statusCode, detailedMsg=""):
        self.detail = detail
        self.status_code = statusCode
        self.default_code = defaultCode
        self.detailed_msg = detailedMsg
        logger.error(self.detailed_msg)


def handle_gateway_exception(f):
    def wrapper(*args, **kwargs):
        obj = args[0]
        try:
            resp = f(*args, **kwargs)
            return resp
        except KubefacetsAPIException as e:
            jsonData = {
                "status": False,
                "data": {},
                "error": {
                    "detail": e.detail,
                    "status_code": e.status_code,
                    "default_code": e.default_code,
                    "type": "error",
                    "detailed_msg": e.detailed_msg,
                },
            }
            resp = requests.Response()
            resp.status_code = status.HTTP_504_GATEWAY_TIMEOUT
            resp._content = json.dumps(jsonData).encode("utf-8")
            resp.headers["Content-Type"] = "application/json"
            obj.setResponse(resp)
            obj.getLogger().logWebRequestError(
                obj.getDjangoRequest().getRequest(),
                jsonData,
                DictUtils.get_by_path(jsonData, "error.type"),
            )
            return resp.json()

    return wrapper



from datetime import datetime, timedelta
from django.utils import timezone
import time

from kfsd.apps.core.exceptions.exec import ExecExceptionHandler
from kfsd.apps.core.common.logger import Logger, LogLevel

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class Time:
    @staticmethod
    @ExecExceptionHandler(logger)
    def sleep(interval):
        time.sleep(interval)

    @staticmethod
    @ExecExceptionHandler(logger)
    def current_time():
        return timezone.now()

    @staticmethod
    @ExecExceptionHandler(logger)
    def future_time(dateconfig, isStrFormat=False, format="%m/%d/%Y, %H:%M:%S %p"):
        dt = Time.current_time()
        return Time.calculate_time(dt, dateconfig, isStrFormat, format)

    @staticmethod
    @ExecExceptionHandler(logger)
    def calculate_time(
        dt, dateconfig, isStrFormat=False, format="%m/%d/%Y, %H:%M:%S %p"
    ):
        td = timedelta(**dateconfig)
        newtime = dt + td
        if not isStrFormat:
            return newtime
        return newtime.strftime(format)

    @staticmethod
    @ExecExceptionHandler(logger)
    def convert_datetime_str(date_str, format="%m/%d/%Y, %H:%M:%S %p"):
        date_obj = datetime.strptime(date_str, format)
        return date_obj



from rest_framework import serializers
from django.utils.translation import gettext_lazy as _
from django.core.validators import (
    MinLengthValidator,
    MaxLengthValidator,
    RegexValidator,
)

from kfsd.apps.models.tables.outpost import Outpost
from kfsd.apps.endpoints.serializers.common.model import BaseModelSerializer
from kfsd.apps.models.constants import MAX_LENGTH, MIN_LENGTH, MSMQ_NAME_REGEX_CONDITION


class MsgQueueSerializer(serializers.Serializer):
    exchange_name = serializers.CharField(
        validators=[
            MinLengthValidator(MIN_LENGTH),
            MaxLengthValidator(MAX_LENGTH),
            RegexValidator(
                MSMQ_NAME_REGEX_CONDITION,
                message=_(
                    "exchange name has to match {}".format(MSMQ_NAME_REGEX_CONDITION)
                ),
                code="exchange_invalid_name",
            ),
        ]
    )
    queue_name = serializers.CharField(
        validators=[
            MinLengthValidator(MIN_LENGTH),
            MaxLengthValidator(MAX_LENGTH),
            RegexValidator(
                MSMQ_NAME_REGEX_CONDITION,
                message=_(
                    "queue name has to match {}".format(MSMQ_NAME_REGEX_CONDITION)
                ),
                code="queue_invalid_name",
            ),
        ]
    )
    routing_key = serializers.CharField(required=False, allow_blank=True)


class MsgSerializer(serializers.Serializer):
    action = serializers.CharField(
        validators=[
            MinLengthValidator(MIN_LENGTH),
            MaxLengthValidator(MAX_LENGTH),
            RegexValidator(
                MSMQ_NAME_REGEX_CONDITION,
                message=_(
                    "action name has to match {}".format(MSMQ_NAME_REGEX_CONDITION)
                ),
                code="action_invalid_name",
            ),
        ],
        required=True,
    )
    data = serializers.JSONField(default=dict)


class OutpostModelSerializer(BaseModelSerializer):
    msg_queue_info = MsgQueueSerializer()
    msg = MsgSerializer()
    status = serializers.ChoiceField(
        choices=["PENDING", "IN-PROGRESS", "ERROR", "COMPLETED"], default="IN-PROGRESS"
    )
    attempts = serializers.IntegerField(default=0)
    debug_info = serializers.JSONField(default=dict, read_only=True)

    class Meta:
        model = Outpost
        fields = "__all__"


class OutpostViewModelSerializer(OutpostModelSerializer):
    id = None
    created = None
    updated = None

    class Meta:
        model = Outpost
        exclude = ("created", "updated", "id")



class CSRF:
    CSRF_HEADER_KEY = "X-CSRFToken"

    def __init__(self):
        self.__csrf = None

    def setCSRF(self, csrf):
        self.__csrf = csrf

    def getCSRF(self):
        return self.__csrf

    def hasCSRF(self):
        if self.__csrf:
            return True
        return False



from drf_spectacular.utils import OpenApiExample, OpenApiTypes, OpenApiParameter


class OutpostV1Doc:
    @staticmethod
    def send_all_view_examples():
        return [
            OpenApiExample(
                "Outpost - Send All",
                value={"detail": "ok"},
                request_only=False,
                response_only=True,
            ),
        ]

    @staticmethod
    def modelviewset_list_path_examples():
        return []

    @staticmethod
    def modelviewset_list_examples():
        return [
            OpenApiExample(
                "Outpost - List All",
                value=[
                    {
                        "identifier": "9fe1f2f2221c6b6a0f28dde862d55299",
                        "msg_queue_info": {
                            "exchange_name": "remind.exchange",
                            "queue_name": "remind.queue",
                            "routing_key": "remind.queue.route.key",
                        },
                        "msg": {
                            "action": "CREATE_REMINDER",
                            "data": {
                                "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                                "type": "ONETIME",
                                "remind_by_in_mins": 10,
                                "to_msg_queue": {
                                    "exchange_name": "certs.exchange",
                                    "queue_name": "certs.queue",
                                    "routing_key": "certs.queue.route.key",
                                },
                                "msg": {
                                    "action": "REORDER_CERT",
                                    "data": {
                                        "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                                    },
                                },
                            },
                        },
                        "status": "IN-PROGRESS",
                        "attempts": 0,
                        "debug_info": {},
                    },
                    {
                        "identifier": "7b7ffebd901976a771f0a51cd7e4aedc",
                        "msg_queue_info": {
                            "exchange_name": "remind.exchange",
                            "queue_name": "remind.queue",
                            "routing_key": "remind.queue.route.key",
                        },
                        "msg": {
                            "action": "CREATE_REMINDER",
                            "data": {
                                "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                                "type": "ONETIME",
                                "remind_by_in_mins": 10,
                                "to_msg_queue": {
                                    "exchange_name": "certs.exchange",
                                    "queue_name": "certs.queue",
                                    "routing_key": "certs.queue.route.key",
                                },
                                "msg": {
                                    "action": "REORDER_CERT",
                                    "data": {
                                        "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                                    },
                                },
                            },
                        },
                        "status": "IN-PROGRESS",
                        "attempts": 0,
                        "debug_info": {},
                    },
                ],
                request_only=False,
                response_only=True,
            )
        ]

    @staticmethod
    def modelviewset_get_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "Outpost - Get",
                        summary="Outpost Identifier",
                        description="Outpost - Get",
                        value="9fe1f2f2221c6b6a0f28dde862d55299",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_get_examples():
        return [
            OpenApiExample(
                "Outpost - Get",
                value={
                    "identifier": "9fe1f2f2221c6b6a0f28dde862d55299",
                    "msg_queue_info": {
                        "exchange_name": "remind.exchange",
                        "queue_name": "remind.queue",
                        "routing_key": "remind.queue.route.key",
                    },
                    "msg": {
                        "action": "CREATE_REMINDER",
                        "data": {
                            "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                            "type": "ONETIME",
                            "remind_by_in_mins": 10,
                            "to_msg_queue": {
                                "exchange_name": "certs.exchange",
                                "queue_name": "certs.queue",
                                "routing_key": "certs.queue.route.key",
                            },
                            "msg": {
                                "action": "REORDER_CERT",
                                "data": {
                                    "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                                },
                            },
                        },
                    },
                    "status": "IN-PROGRESS",
                    "attempts": 0,
                    "debug_info": {},
                },
                request_only=False,
                response_only=True,
            )
        ]

    @staticmethod
    def modelviewset_create_examples():
        return [
            OpenApiExample(
                "Outpost - Create",
                value={
                    "msg_queue_info": {
                        "exchange_name": "remind.exchange",
                        "queue_name": "remind.queue",
                        "routing_key": "remind.queue.route.key",
                    },
                    "msg": {
                        "action": "CREATE_REMINDER",
                        "data": {
                            "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                            "type": "ONETIME",
                            "remind_by_in_mins": 10,
                            "to_msg_queue": {
                                "exchange_name": "certs.exchange",
                                "queue_name": "certs.queue",
                                "routing_key": "certs.queue.route.key",
                            },
                            "msg": {
                                "action": "REORDER_CERT",
                                "data": {
                                    "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                                },
                            },
                        },
                    },
                },
                request_only=True,
                response_only=False,
            ),
            OpenApiExample(
                "Outpost - Create",
                value={
                    "identifier": "9fe1f2f2221c6b6a0f28dde862d55299",
                    "msg_queue_info": {
                        "exchange_name": "remind.exchange",
                        "queue_name": "remind.queue",
                        "routing_key": "remind.queue.route.key",
                    },
                    "msg": {
                        "action": "CREATE_REMINDER",
                        "data": {
                            "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                            "type": "ONETIME",
                            "remind_by_in_mins": 10,
                            "to_msg_queue": {
                                "exchange_name": "certs.exchange",
                                "queue_name": "certs.queue",
                                "routing_key": "certs.queue.route.key",
                            },
                            "msg": {
                                "action": "REORDER_CERT",
                                "data": {
                                    "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                                },
                            },
                        },
                    },
                    "status": "IN-PROGRESS",
                    "attempts": 0,
                    "debug_info": {},
                },
                request_only=False,
                response_only=True,
            ),
        ]

    @staticmethod
    def modelviewset_delete_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "Outpost - Delete",
                        summary="Outpost Identifier",
                        description="Outpost - Delete",
                        value="9fe1f2f2221c6b6a0f28dde862d55299",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_patch_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "Outpost - Delete",
                        summary="Outpost Identifier",
                        description="Outpost - Delete",
                        value="9fe1f2f2221c6b6a0f28dde862d55299",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_patch_examples():
        return [
            OpenApiExample(
                "Outpost - Patch",
                value={
                    "msg_queue_info": {
                        "exchange_name": "remind.exchange1",
                        "queue_name": "remind.queue",
                        "routing_key": "remind.queue.route.key",
                    },
                },
                request_only=True,
                response_only=False,
            ),
            OpenApiExample(
                "Outpost - Patch",
                value={
                    "identifier": "9fe1f2f2221c6b6a0f28dde862d55299",
                    "msg_queue_info": {
                        "exchange_name": "remind.exchange1",
                        "queue_name": "remind.queue",
                        "routing_key": "remind.queue.route.key",
                    },
                    "msg": {
                        "action": "CREATE_REMINDER",
                        "data": {
                            "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                            "type": "ONETIME",
                            "remind_by_in_mins": 10,
                            "to_msg_queue": {
                                "exchange_name": "certs.exchange",
                                "queue_name": "certs.queue",
                                "routing_key": "certs.queue.route.key",
                            },
                            "msg": {
                                "action": "REORDER_CERT",
                                "data": {
                                    "identifier": "ORG=Kubefacets,APP=Certs,PRJ=Auth,COLL=Login,CSR=kubefacets Root CA",
                                },
                            },
                        },
                    },
                    "status": "IN-PROGRESS",
                    "attempts": 0,
                    "debug_info": {},
                },
                request_only=False,
                response_only=True,
            ),
        ]



from django.urls import path, re_path
from django.views.generic.base import RedirectView
from kfsd.apps.frontend.views.docs.api import APIDocsView

urlpatterns = [
    re_path(r'^$', RedirectView.as_view(url='apis/doc/', permanent=False), name='api_doc'),
    path('apis/doc/', APIDocsView.as_view(), name="api_doc"),
]



from kfsd.apps.frontend.permissions.base import BasePermission
from kfsd.apps.core.exceptions.fe import KubefacetsFEException
from http import HTTPStatus


class IsActive(BasePermission):
    def __init__(self, request):
        BasePermission.__init__(self, request)

    def __str__(self):
        return "Is Active check"

    def is_valid(self):
        if self.getUser().isActive():
            return True
        return False

    def raise_exception(self):
        raise KubefacetsFEException(self.__str__(), HTTPStatus.FORBIDDEN, None)

    def redirect_url(self):
        return ""

    def redirect_url_neg(self):
        return ""

    def raise_exception_neg(self):
        raise KubefacetsFEException(self.__str__(), HTTPStatus.FORBIDDEN, None)



from rest_framework import viewsets, filters
from kfsd.apps.endpoints.renderers.kubefacetsjson import KubefacetsJSONRenderer
from kfsd.apps.endpoints.views.pagination.model import ModelPagination


class ModelViewSet(viewsets.ModelViewSet):
    http_method_names = ["get", "post", "patch", "delete"]
    renderer_classes = [KubefacetsJSONRenderer]
    pagination_class = ModelPagination
    filter_backends = [filters.OrderingFilter]
    ordering = ["created"]



from rest_framework import serializers
from kfsd.apps.endpoints.serializers.base import BaseInputReqSerializer, BaseOutputRespSerializer


class SystemInputReqSerializer(BaseInputReqSerializer):
    op = serializers.ChoiceField(
        choices=[
            "CHECKSUM", "UUID", "SECRET", "KEY", "ENCRYPT_KEY", "OS_ARCH", "HOST_IP", "NIC", "HOSTNAME", "OS"
        ]
    )


class SystemOutputRespSerializer(BaseOutputRespSerializer):
    op = serializers.ChoiceField(
        choices=[
            "CHECKSUM", "UUID", "SECRET", "KEY", "ENCRYPT_KEY", "OS_ARCH", "HOST_IP", "NIC", "HOSTNAME", "OS"
        ]
    )



from drf_spectacular.utils import extend_schema

from kfsd.apps.endpoints.views.docs.v1.outpost import OutpostV1Doc
from kfsd.apps.endpoints.serializers.common.outpost import OutpostViewModelSerializer
from kfsd.apps.endpoints.serializers.base import (
    NotFoundSerializer,
    ErrorSerializer,
    SuccessSerializer,
)


class OutpostDoc:
    @staticmethod
    def modelviewset():
        return {
            "list": extend_schema(**OutpostDoc.modelviewset_list()),
            "retrieve": extend_schema(**OutpostDoc.modelviewset_get()),
            "destroy": extend_schema(**OutpostDoc.modelviewset_delete()),
            "partial_update": extend_schema(**OutpostDoc.modelviewset_patch()),
            "create": extend_schema(**OutpostDoc.modelviewset_create()),
        }

    @staticmethod
    def send_all_view():
        return {
            "summary": "Outpost - Send All",
            "tags": ["OUTPOST"],
            "responses": {
                200: SuccessSerializer,
                401: ErrorSerializer,
                403: ErrorSerializer,
                500: ErrorSerializer,
            },
            "examples": OutpostV1Doc.send_all_view_examples(),
        }

    @staticmethod
    def msmq_signal_failed():
        return {
            "summary": "Outpost - MSMQ Failed",
            "description": "Outpost MSMQ Failed",
            "tags": ["OUTPOST"],
            "responses": {
                200: OutpostViewModelSerializer,
                404: NotFoundSerializer,
                500: ErrorSerializer,
            },
            "parameters": OutpostV1Doc.modelviewset_patch_path_examples(),
            "examples": OutpostV1Doc.modelviewset_patch_examples(),
        }

    @staticmethod
    def modelviewset_patch():
        return {
            "summary": "Outpost - Patch",
            "description": "Outpost Patch",
            "tags": ["OUTPOST"],
            "responses": {
                200: OutpostViewModelSerializer,
                404: NotFoundSerializer,
                500: ErrorSerializer,
            },
            "parameters": OutpostV1Doc.modelviewset_patch_path_examples(),
            "examples": OutpostV1Doc.modelviewset_patch_examples(),
        }

    @staticmethod
    def modelviewset_list():
        return {
            "summary": "Outpost - List",
            "description": "Outpost - All",
            "tags": ["OUTPOST"],
            "responses": {
                200: OutpostViewModelSerializer,
                404: NotFoundSerializer,
                500: ErrorSerializer,
            },
            "parameters": OutpostV1Doc.modelviewset_list_path_examples(),
            "examples": OutpostV1Doc.modelviewset_list_examples(),
        }

    @staticmethod
    def modelviewset_get():
        return {
            "summary": "Outpost - Get",
            "description": "Outpost Detail",
            "tags": ["OUTPOST"],
            "responses": {
                200: OutpostViewModelSerializer,
                404: NotFoundSerializer,
                500: ErrorSerializer,
            },
            "parameters": OutpostV1Doc.modelviewset_get_path_examples(),
            "examples": OutpostV1Doc.modelviewset_get_examples(),
        }

    @staticmethod
    def modelviewset_create():
        return {
            "summary": "Outpost - Create",
            "description": "Outpost - Create",
            "tags": ["OUTPOST"],
            "responses": {
                200: OutpostViewModelSerializer,
                400: ErrorSerializer,
                404: ErrorSerializer,
                500: ErrorSerializer,
            },
            "examples": OutpostV1Doc.modelviewset_create_examples(),
        }

    @staticmethod
    def modelviewset_delete():
        return {
            "summary": "Outpost - Delete",
            "description": "Outpost Delete",
            "tags": ["OUTPOST"],
            "responses": {204: None, 404: NotFoundSerializer, 500: ErrorSerializer},
            "parameters": OutpostV1Doc.modelviewset_delete_path_examples(),
        }



from kfsd.apps.core.utils.http.base import HTTP
from kfsd.apps.core.utils.http.django.request import DjangoRequest
from kfsd.apps.core.utils.http.django.config import DjangoConfig
from kfsd.apps.core.common.kubefacets_config import KubefacetsConfig


class BasePermission(HTTP):
    def __init__(self, request):
        self.__request = DjangoRequest(request)
        self.__config = DjangoConfig(KubefacetsConfig().getConfig())
        HTTP.__init__(self)

    def getDjangoRequest(self):
        return self.__request

    def getDjangoConfig(self):
        return self.__config

    def getUser(self):
        return self.getDjangoRequest().getRequest().token_user

    def getCurrentRequestUrl(self):
        return self.getDjangoRequest().getRequest().build_absolute_uri()



import os
import shutil
import json
from kfsd.apps.core.common.logger import Logger, LogLevel
from kfsd.apps.core.exceptions.exec import ExecExceptionHandler

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class FileUtils():

    @staticmethod
    @ExecExceptionHandler(logger)
    def read_as_json(path):
        with open(path) as json_file:
            return json.load(json_file)

    @staticmethod
    def path_exists(destPath):
        if not os.path.exists(destPath):
            return False
        return True

    @staticmethod
    def construct_path(dir, dest):
        return os.path.join(dir, dest)

    @staticmethod
    @ExecExceptionHandler(logger)
    def copy_dir(src, dest):
        if not FileUtils.path_exists(dest):
            return shutil.copytree(src, dest)

    @staticmethod
    @ExecExceptionHandler(logger)
    def create_dir(dirPath):
        if not FileUtils.path_exists(dirPath):
            os.makedirs(dirPath)
            return True
        return False

    @staticmethod
    @ExecExceptionHandler(logger)
    def read_as_text(filePath, removeLineBreaks=True):
        fileStr = ""
        with open(filePath, 'r') as file:
            fileStr = file.read()
        return fileStr.replace("\n", " ") if removeLineBreaks else fileStr

    @staticmethod
    @ExecExceptionHandler(logger)
    def read(filePath, startLineNum=0, endLineNum=None, removeLineBreaks=True):
        lines = []
        with open(filePath, 'r') as file:
            lines = file.readlines()
        return [x.replace("\n", "") for x in lines][startLineNum:endLineNum] if removeLineBreaks else lines[startLineNum:endLineNum]

    @staticmethod
    @ExecExceptionHandler(logger)
    def write(filePath, data):
        with open(filePath, 'w') as file:
            file.write(data)
            file.close()

    @staticmethod
    @ExecExceptionHandler(logger)
    def write_as_json(jsonFilePath, dictData):
        with open(jsonFilePath, 'w') as jsonFile:
            json.dump(dictData, jsonFile)

    @staticmethod
    def get_dir_from_filepath(filePath):
        return os.path.dirname(filePath)

    @staticmethod
    def create_target_dir(filePath):
        destDir = FileUtils.get_dir_from_filepath(filePath)
        FileUtils.create_dir(destDir)

    @staticmethod
    @ExecExceptionHandler(logger)
    def move_file(srcFile, destFile):
        FileUtils.create_target_dir(destFile)
        shutil.move(srcFile, destFile)

    @staticmethod
    @ExecExceptionHandler(logger)
    def copy_file(srcFile, destFilePath):
        FileUtils.create_target_dir(destFilePath)
        return shutil.copy(srcFile, destFilePath)

    @staticmethod
    @ExecExceptionHandler(logger)
    def rm_dir(dirPath):
        if FileUtils.path_exists(dirPath):
            shutil.rmtree(dirPath)
            return True
        return False

    @staticmethod
    @ExecExceptionHandler(logger)
    def rm_file(filePath):
        if FileUtils.path_exists(filePath):
            os.remove(filePath)
            return True



class APIKey:
    APIKEY_HEADER_KEY = "X-APIKey"

    def __init__(self):
        self.__apiKey = None

    def setAPIKey(self, apiKey):
        self.__apiKey = apiKey

    def getAPIKey(self):
        return self.__apiKey

    def hasAPIKey(self):
        if self.__apiKey:
            return True
        return False



from drf_spectacular.utils import extend_schema_view, extend_schema
from rest_framework.decorators import action
from rest_framework.renderers import JSONRenderer
from rest_framework.views import Response

from kfsd.apps.models.tables.outpost import Outpost, send_msg
from kfsd.apps.endpoints.serializers.common.outpost import OutpostViewModelSerializer
from kfsd.apps.endpoints.views.docs.outpost import OutpostDoc
from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet


@extend_schema_view(**OutpostDoc.modelviewset())
class OutpostModelViewSet(CustomModelViewSet):
    queryset = Outpost.objects.all()
    serializer_class = OutpostViewModelSerializer

    @extend_schema(**OutpostDoc.send_all_view())
    @action(
        detail=False,
        methods=["get"],
        renderer_classes=[JSONRenderer],
        url_path="send/all",
    )
    def send_all(self, request, *args, **kwargs):
        queryset = Outpost.objects.all()
        for obj in queryset:
            send_msg.delay(obj.id)
        return Response({"detail": "ok"})



from kfsd.apps.endpoints.handlers.common.base import BaseHandler
from kfsd.apps.endpoints.serializers.common.outpost import OutpostModelSerializer
from kfsd.apps.models.tables.outpost import Outpost


class OutpostHandler(BaseHandler):
    def __init__(self, outpostIdentifier, isDBFetch):
        BaseHandler.__init__(
            self,
            serializer=OutpostModelSerializer,
            modelClass=Outpost,
            identifier=outpostIdentifier,
            isDBFetch=isDBFetch,
        )

    def updateStatus(self, status):
        pass



import zlib
import json
import secrets
import binascii
import time
import subprocess
import shlex
import netifaces
import socket
import os

from kfsd.apps.core.common.logger import Logger, LogLevel
from kfsd.apps.core.exceptions.exec import ExecExceptionHandler
from kfsd.apps.core.utils.file import FileUtils

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class System:
    def __init__(self, **kwargs):
        self.__cmdExecStatus = False
        self.__cmdExecOutput = None

    def setCmdExecStatus(self, cmdStatus):
        self.__cmdExecStatus = cmdStatus

    def getCmdExecStatus(self):
        return self.__cmdExecStatus

    def setCmdExecOutput(self, cmdOutput):
        self.__cmdExecOutput = cmdOutput

    def getCmdExecOutput(self):
        return self.__cmdExecOutput

    def encryptKey(self, key=None):
        if not key:
            self.cmdExec("head -c 32 /dev/urandom | base64")
        else:
            self.cmdExec("echo {} | base64".format(key))
        return self.getCmdExecOutput()

    def osArch(self):
        self.cmdExec("uname -m")
        return self.getCmdExecOutput().lower()

    def hostIP(self):
        return netifaces.ifaddresses(self.getNIC())[netifaces.AF_INET][0]["addr"]

    def getNIC(self):
        nicPath = "/sys/class/net/{}/"
        if FileUtils.path_exists(nicPath.format("eth0")):
            return "eth0"
        elif FileUtils.path_exists(nicPath.format("enp5s0")):
            return "enp5s0"
        return "en0"

    def getHostName(self):
        return socket.gethostname()

    def getOS(self):
        self.cmdExec("uname")
        return self.getCmdExecOutput()

    def changeFilePermission(self, permission, filePath):
        self.cmdExec("chmod {} {}".format(permission, filePath))
        return self.getCmdExecStatus()

    def cmdsExec(self, cmds, captureOutput=True, shell=False):
        return [
            self.cmdExec(cmd, captureOutput, shell)
            if not type(cmd) == list
            else self.cmdsExec(cmd, captureOutput, shell)
            for cmd in cmds
        ]

    def cmdExec(self, cmd, captureOutput=True, shell=False):
        if captureOutput:
            cmdOutput = subprocess.getstatusoutput(cmd)
            cmdExecStatus = True if cmdOutput[0] == 0 else False
            cmdOutput = cmdOutput[1]
            self.setCmdExecStatus(cmdExecStatus)
            self.setCmdExecOutput(cmdOutput)
        else:
            logger.debug("CMD: {}".format(cmd))
            proc = subprocess.Popen(
                shlex.split(cmd) if not shell else cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                shell=shell,
                universal_newlines=True,
            )
            for line in iter(proc.stdout.readline, ""):
                logger.debug("{}".format(line.strip()))
            proc.stdout.close()

    @staticmethod
    @ExecExceptionHandler(logger)
    def sleep(seconds):
        time.sleep(seconds)

    @staticmethod
    def getEnv(envKey):
        return os.getenv(envKey)

    @staticmethod
    @ExecExceptionHandler(logger)
    def checksum(data):
        formattedData = None
        if isinstance(data, str):
            formattedData = data.encode("utf-8")
        elif isinstance(data, list) or isinstance(data, dict):
            formattedData = json.dumps(data).encode("utf-8")
        else:
            formattedData = bytes(data)

        return zlib.adler32(formattedData) & 0xFFFFFFFF

    @staticmethod
    @ExecExceptionHandler(logger)
    def uuid(len):
        return secrets.token_hex(int(len / 2))

    @staticmethod
    @ExecExceptionHandler(logger)
    def secret(len):
        return secrets.token_urlsafe(len)

    @staticmethod
    @ExecExceptionHandler(logger)
    def api_key(len):
        api_key_bytes = secrets.token_bytes(int(len / 2))
        return binascii.hexlify(api_key_bytes).decode("utf-8")



from rest_framework.views import Response, exception_handler
from rest_framework import status
import json
import traceback
from django.conf import settings
from rest_framework.exceptions import ValidationError
from django.db import IntegrityError
from django.core.exceptions import ValidationError as CoreValidationError
from kfsd.apps.core.exceptions.api import KubefacetsAPIException


def KubefacetsAPIExceptionHandler(ex, context):
    response = exception_handler(ex, context)
    request = context["request"]
    errorData = {
        "status": "ERROR",
        "path": request.path,
        "method": request.method,
        "content_type": request.content_type,
        "query_params": request.query_params,
        "body": request.data,
        "cookies": request.COOKIES,
        "error": ex.__str__(),
    }
    errorJson = json.dumps(errorData, indent=4)
    print(errorJson)

    # print error to console
    if settings.DEBUG:
        print("[[ STACKTRACE ]]")
        traceback.print_exc()

    if isinstance(ex, IntegrityError) and not response:
        response = Response(
            {"detail": ex.__str__(), "code": "bad_request"},
            status=status.HTTP_400_BAD_REQUEST,
        )

    if isinstance(ex, TypeError) and not response:
        response = Response(
            {"detail": ex.__str__(), "code": "system_error"},
            status=status.HTTP_500_INTERNAL_SERVER_ERROR,
        )

    if isinstance(ex, ValidationError):
        response = Response(
            {"detail": ex.args[0], "code": "bad_request"},
            status=status.HTTP_400_BAD_REQUEST,
        )

    if isinstance(ex, CoreValidationError):
        response = Response(
            {"detail": ex.args[0], "code": "bad_request"},
            status=status.HTTP_400_BAD_REQUEST,
        )

    if isinstance(ex, KubefacetsAPIException):
        response = Response(
            {"detail": ex.detail, "code": ex.default_code}, status=ex.status_code
        )
    return response



from kfsd.apps.frontend.permissions.base import BasePermission
from kfsd.apps.core.exceptions.fe import KubefacetsFEException
from http import HTTPStatus


class SignInRequired(BasePermission):
    def __init__(self, request):
        BasePermission.__init__(self, request)

    def __str__(self):
        return "Is authenticated check"

    def is_valid(self):
        if self.getUser().isAuthenticated():
            return True
        return False

    def raise_exception(self):
        redirectUrl = self.redirect_url()
        raise KubefacetsFEException(
            self.__str__(), HTTPStatus.TEMPORARY_REDIRECT, redirectUrl
        )

    def redirect_url(self):
        loginUrl = self.formatUrl(
            self.getDjangoRequest().findConfigs(
                ["services.sso_fe.host", "services.sso_fe.signin_uri"]
            )
        )
        loginUrl += "?next={}".format(self.genUrlEncode(self.getCurrentRequestUrl()))
        return loginUrl

    def redirect_url_neg(self):
        verifiedFinalUrl = self.formatUrl(
            self.getDjangoRequest().findConfigs(["services.default_signedin_url"])
        )
        return verifiedFinalUrl

    def raise_exception_neg(self):
        raise KubefacetsFEException(
            self.__str__(), HTTPStatus.TEMPORARY_REDIRECT, self.redirect_url_neg()
        )



import requests
from django.http import HttpRequest

from kfsd.apps.core.utils.http.django.cookie import Cookie
from kfsd.apps.core.utils.http.django.config import DjangoConfig


class DjangoRequest(DjangoConfig):
    def __init__(self, request=None):
        self.__request = request
        self.__djangoCookies = Cookie(request)
        DjangoConfig.__init__(self, self.getConfigData())

    def getRequest(self):
        return self.__request

    def getDjangoReqCookies(self):
        return self.__djangoCookies

    def parseInputData(self, serializer, raiseExceptions=True):
        inputSerializer = serializer(data=self.__request.data)
        inputSerializer.is_valid(raise_exception=raiseExceptions)
        return inputSerializer.data

    def getConfigData(self):
        return self.__request.config

    @staticmethod
    def genDjangoRequest(url, method):
        request = requests.Request(method, url)
        prepared_request = request.prepare()

        django_request = HttpRequest()
        django_request.method = prepared_request.method
        django_request.path = prepared_request.path_url
        return django_request



from kfsd.apps.core.utils.dict import DictUtils


class ArrUtils:

    # items that are common between both arrays
    @staticmethod
    def intersection(arr1, arr2):
        return list((set(arr1) & set(arr2)))

    # merge 2 lists
    @staticmethod
    def merge(arr1, arr2, arr_rm_dupes=True, arr_dict_lookup_key="identifier", is_arr_sorted=True):
        mergedArr = arr1 + arr2
        return DictUtils.match_and_merge_arr(mergedArr, arr_rm_dupes, arr_dict_lookup_key, is_arr_sorted)

    # join array using join_str
    @staticmethod
    def join(arr, join_str):
        return join_str.join(map(str, arr))

    @staticmethod
    def sort_keys_by_len(keys, isReverse=False):
        return sorted(keys, key=len, reverse=isReverse)

    @staticmethod
    def to_dict(arr, key):
        return {item[key]: item for item in arr}



import json


def Singleton(theClass):
    classInstances = {}

    def getInstance(*args, **kwargs):
        key = (theClass, args, json.dumps(kwargs))
        if key not in classInstances:
            classInstances[key] = theClass(*args, **kwargs)
        return classInstances[key]
    return getInstance



from kfsd.apps.core.tests.base_api import BaseAPITestCases
from django.urls import reverse
from unittest.mock import patch


class APIDocsViewTests(BaseAPITestCases):
    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getLocalKubefacetsSettingsConfig"
    )
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.deriveRemoteConfig"
    )
    def test_get(self, remoteConfigMocked, rawSettingsMocked, tokenUserInfoMocked):
        rawConfig = self.readJSONData(
            "kfsd/apps/core/tests/v1/data/requests/common/kubefacets/remote_settings.json"
        )
        remoteConfig = self.readJSONData(
            "kfsd/apps/core/tests/v1/data/requests/common/kubefacets/remote_config_resp.json"
        )
        rawSettingsMocked.return_value = rawConfig
        remoteConfigMocked.return_value = remoteConfig
        staffUserInfoResp = {
            "status": True,
            "data": {
                "user": {
                    "identifier": "123",
                    "is_staff": True,
                    "is_active": True,
                    "is_email_verified": True,
                }
            },
        }
        tokenUserInfoMocked.return_value = staffUserInfoResp
        url = reverse("api_doc")
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)



from django.views.generic import View
from http import HTTPStatus
from django.http import (
    HttpResponseRedirect,
    HttpResponseNotFound,
    HttpResponseServerError,
    HttpResponseForbidden,
)

from kfsd.apps.core.exceptions.fe import KubefacetsFEException
from kfsd.apps.core.exceptions.api import KubefacetsAPIException


class PermissionView(View):
    def verifyPermission(self, permission):
        if not permission.is_valid():
            permission.raise_exception()
        return True

    def verifyPermissionNeg(self, permission):
        if permission.is_valid():
            permission.raise_exception_neg()
        return True

    def getPermissionsClasses(self, argName):
        if hasattr(self, argName):
            return getattr(self, argName)
        return []

    def checkAllPermissions(self, request):
        for permission in self.getPermissionsClasses("permission_classes"):
            permissionObj = permission(request)
            self.verifyPermission(permissionObj)
        return True

    def checkAllPermissionsNeg(self, request):
        for permission in self.getPermissionsClasses("permission_classes_neg"):
            permissionObj = permission(request)
            self.verifyPermissionNeg(permissionObj)
        return True

    def dispatch(self, request, *args, **kwargs):
        try:
            self.checkAllPermissions(request)
            self.checkAllPermissionsNeg(request)
            return super().dispatch(request, *args, **kwargs)
        except KubefacetsFEException as ex:
            if ex.status_code == HTTPStatus.TEMPORARY_REDIRECT:
                return HttpResponseRedirect(ex.redirect_url)
            if ex.status_code == HTTPStatus.NOT_FOUND:
                return HttpResponseNotFound()
            if ex.status_code == HTTPStatus.FORBIDDEN:
                return HttpResponseForbidden()
            return HttpResponseServerError()
        except KubefacetsAPIException:
            return HttpResponseServerError()



from kfsd.apps.core.common.configuration import Configuration


class DjangoConfig:
    def __init__(self, config={}):
        self.__config = config

    def getConfig(self):
        return self.__config

    def findConfigs(self, paths):
        return Configuration.findConfigValues(self.getConfig(), paths)



from unittest.mock import patch

from kfsd.apps.core.tests.base_api import BaseAPITestCases
from kfsd.apps.core.common.kubefacets_config import KubefacetsConfig

middleware_settings = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "kfsd.apps.core.middleware.token.KubefacetsTokenMiddleware",
]


class KubefacetsConfigTests(BaseAPITestCases):
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getLocalKubefacetsSettingsConfig"
    )
    def test_kubefacets_config_local(self, rawSettingsMocked):
        rawConfig = self.readJSONData(
            "kfsd/apps/core/tests/v1/data/requests/common/kubefacets/local_settings.json"
        )
        rawSettingsMocked.return_value = rawConfig
        obsConfig = KubefacetsConfig().getConfig()
        expConfig = self.readJSONData(
            "kfsd/apps/core/tests/v1/data/responses/common/kubefacets/test_kubefacets_config_local.json"
        )
        self.assertEquals(obsConfig, expConfig)

        localObsConfig = KubefacetsConfig().getLocalConfig()
        localExpConfig = self.readJSONData(
            "kfsd/apps/core/tests/v1/data/responses/common/kubefacets/test_kubefacets_config_local_localconfigobj.json"
        )
        self.assertEquals(localObsConfig, localExpConfig)

    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getLocalKubefacetsSettingsConfig"
    )
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.deriveRemoteConfig"
    )
    def test_kubefacets_config_remote(self, remoteConfigMocked, rawSettingsMocked):
        rawConfig = self.readJSONData(
            "kfsd/apps/core/tests/v1/data/requests/common/kubefacets/remote_settings.json"
        )
        remoteConfig = self.readJSONData(
            "kfsd/apps/core/tests/v1/data/requests/common/kubefacets/remote_config_resp.json"
        )
        rawSettingsMocked.return_value = rawConfig
        remoteConfigMocked.return_value = remoteConfig
        obsConfig = KubefacetsConfig().getConfig()
        expConfig = self.readJSONData(
            "kfsd/apps/core/tests/v1/data/responses/common/kubefacets/test_kubefacets_config_remote.json"
        )
        self.assertEquals(obsConfig, expConfig)



from kfsd.apps.core.services.gateway.sso import SSO
from kfsd.apps.core.common.logger import Logger, LogLevel

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class TokenAuth(SSO):
    def __init__(self, request=None):
        SSO.__init__(self, request)

    def getTokenUserInfo(self):
        payload = {
            "cookies": self.getDjangoRequest().getDjangoReqCookies().getAllCookies()
        }
        return self.verifyTokens(payload)



from rest_framework import serializers


class BaseModelSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(read_only=True)
    identifier = serializers.CharField(read_only=True)
    created = serializers.DateTimeField(read_only=True)
    updated = serializers.DateTimeField(read_only=True)
    slug = serializers.SlugField(read_only=True)



from kfsd.apps.core.exceptions.api import KubefacetsAPIException
from kfsd.apps.core.utils.http.headers.cookie import Cookie
from kfsd.apps.core.utils.dict import DictUtils


class Response(Cookie):
    def __init__(self):
        self.__response = None

    def getResponse(self):
        return self.__response

    def getJsonResponse(self):
        return self.__response.json()

    def setResponse(self, response):
        self.__response = response
        self.setCookiesHttpObj(response)

    def getStatusCode(self):
        return self.__response.status_code

    def isRespJSON(self):
        if (
            "application/json"
            in self.__response.headers.get("content-type", "").lower()
        ):
            return True
        return False

    def raiseAPIException(self):
        resp = self.getJsonResponse() if self.isRespJSON() else {}
        errorStr = DictUtils.get(resp, "detail", "Unknown error detail")
        errorCode = DictUtils.get(resp, "code", "unexpected_error")
        raise KubefacetsAPIException(errorStr, errorCode, self.getStatusCode())

    def isRespValid(self, expStatusCode):
        if isinstance(expStatusCode, int) and not expStatusCode == self.getStatusCode():
            self.raiseAPIException()

        if (
            isinstance(expStatusCode, list)
            and self.getStatusCode() not in expStatusCode
        ):
            self.raiseAPIException()

        return True



from drf_spectacular.utils import extend_schema
from rest_framework import viewsets, decorators, response, status

from kfsd.apps.endpoints.views.docs.common import CommonDoc
from kfsd.apps.endpoints.handlers.common.configuration import ConfigurationHandler
from kfsd.apps.endpoints.serializers.common.configuration import ConfigurationInputReqSerializer
from kfsd.apps.endpoints.renderers.kubefacetsjson import KubefacetsJSONRenderer
from kfsd.apps.endpoints.renderers.kubefacetsyaml import KubefacetsYAMLRenderer


class CommonViewSet(viewsets.ViewSet):
    lookup_field = "identifier"
    lookup_value_regex = '[^/]+'

    def parseInput(self, request, serializer):
        inputSerializer = serializer(data=request.data)
        inputSerializer.is_valid()
        return inputSerializer.data

    def getConfigurationInputData(self, request):
        return self.parseInput(request, ConfigurationInputReqSerializer)

    @extend_schema(**CommonDoc.config_view())
    @decorators.action(detail=False, methods=['post'], renderer_classes=[KubefacetsJSONRenderer, KubefacetsYAMLRenderer])
    def config(self, request):
        configHandler = ConfigurationHandler(self.getConfigurationInputData(request))
        return response.Response(configHandler.gen(), status.HTTP_200_OK)



from rest_framework import status
from django.urls import reverse
from unittest.mock import patch

from kfsd.apps.endpoints.tests.endpoints_test_handler import EndpointsTestHandler


class AttrUtilsTestCases(EndpointsTestHandler):
    def setUp(self):
        super().setUp()

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch("kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getConfig")
    def test_utils_expr(self, genCommonConfigMocked, tokenUserInfoMocked):
        genCommonConfigMocked.side_effect = [{}]
        tokenUserInfoMocked.side_effect = [{}]
        postData = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/requests/utils/attr/test_utils_expr.json')
        obsResponse = self.post(reverse("utils-attr"), postData, status.HTTP_200_OK)
        expResponse = self.readJSONData('kfsd/apps/endpoints/tests/v1/data/responses/utils/attr/test_utils_expr.json')
        self.assertEqual(obsResponse, expResponse)



from django.conf import settings
from rest_framework import status

from kfsd.apps.core.common.configuration import Configuration
from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.utils.system import System
from kfsd.apps.core.utils.http.headers.contenttype import ContentType
from kfsd.apps.core.common.cache import cache
from kfsd.apps.core.exceptions.api import KubefacetsAPIException
from kfsd.apps.core.utils.http.django.request import DjangoRequest
from kfsd.apps.core.auth.api.gateway import APIGateway


class KubefacetsConfig(APIGateway):
    def __init__(self):
        self.__localConfig = self.genLocalConfig()
        APIGateway.__init__(self, self.genRequest())

    def genRequest(self):
        self.setHttpHeaders()
        request = DjangoRequest.genDjangoRequest("http://commonconfig/request", "GET")
        request.config = self.__localConfig
        return request

    def getLocalConfig(self):
        return self.__localConfig

    def getLocalKubefacetsSettingsConfig(self):
        return settings.KUBEFACETS

    def isLocalConfig(self):
        isLocalConfig = DictUtils.get_by_path(self.getLocalKubefacetsSettingsConfig(), "config.is_local_config")
        if isLocalConfig:
            return True
        return False

    def getLocalConfigDimensions(self):
        localConfigLookupDimensions = DictUtils.get_by_path(self.getLocalKubefacetsSettingsConfig(), "config.lookup_dimension_keys")
        if not localConfigLookupDimensions:
            return []
        return localConfigLookupDimensions

    def genLocalConfig(self):
        dimensions = self.constructDimensionsFromEnv(self.getLocalConfigDimensions())
        localConfig = DictUtils.get_by_path(self.getLocalKubefacetsSettingsConfig(), "config.local")
        return Configuration(settings=localConfig, dimensions=dimensions).getFinalConfig()

    @cache("kfsd.gateway.common.config")
    def deriveRemoteConfig(self):
        if self.isLocalConfig():
            return {}

        self.setHttpHeaders()
        resp = self.get(
            self.getCommonConfigUrl(),
            status.HTTP_200_OK,
            headers=self.getReqHeaders(),
        )
        gatewayResp = resp.json()
        if DictUtils.get(gatewayResp, "status"):
            return DictUtils.get(gatewayResp, "data")
        else:
            raise KubefacetsAPIException("Getting Common Config error", "config_error", status.HTTP_500_INTERNAL_SERVER_ERROR, DictUtils.get(gatewayResp, "error"))

    def deriveLocalConfig(self):
        commonConfig = DictUtils.get(self.__localConfig, "common", {})
        return DictUtils.merge(
            dict1=DictUtils.deepcopy(self.__localConfig),
            dict2=DictUtils.deepcopy(commonConfig),
        )

    def getConfig(self):
        if self.isLocalConfig():
            config = self.deriveLocalConfig()
            return config
        else:
            try:
                remoteConfig = self.deriveRemoteConfig()
                remoteConfigMerged = DictUtils.merge(
                    dict1=DictUtils.deepcopy(self.__localConfig),
                    dict2=DictUtils.deepcopy(remoteConfig),
                )
                return remoteConfigMerged
            except KubefacetsAPIException:
                return self.__localConfig

    def constructUrl(self, configPaths):
        uris = self.findConfigs(configPaths)
        return self.formatUrl(uris)

    def findConfigs(self, paths):
        return Configuration.findConfigValues(self.__localConfig, paths)

    def getServicesAPIKey(self):
        return self.findConfigs(["services.api_key"])[0]

    def setHttpHeaders(self):
        self.setAPIKey(self.getServicesAPIKey())
        self.setContentType(ContentType.APPLICATION_JSON)

    def getCommonConfigUrl(self):
        return self.constructUrl(["services.gateway.host", "services.gateway.core.common_config_uri"])

    def constructDimensionsFromEnv(self, dimensionKeys):
        return {key: System.getEnv(key) for key in dimensionKeys}



from rest_framework import generics
from rest_framework.views import Response


class ListAPIView(generics.GenericAPIView):
    def post(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)



import functools

from kfsd.apps.core.utils.arr import ArrUtils
from kfsd.apps.core.utils.dict import DictUtils


class Configuration:
    def __init__(self, **kwargs):
        self.__rawSettings = {
            tuple(item["setting"]): item
            for item in DictUtils.deepcopy(kwargs.pop("settings"))
        }
        self.__reqKeyList = [
            "{}:{}".format(k, v) for (k, v) in kwargs.pop("dimensions").items()
        ]
        self.__mergeRecursive = kwargs.get("merge_recursive", True)
        self.__arr_rm_dupes = kwargs.get("arr_rm_dupes", True)
        self.__config = self.generateConfig()

    @staticmethod
    def findConfigValues(config, paths):
        configs = [DictUtils.get_by_path(config, path) for path in paths]
        return configs

    def getFinalConfig(self):
        return self.__config

    def findSetting(self, key):
        tupleKey = tuple(key)
        return (
            {} if tupleKey not in self.__rawSettings else self.__rawSettings[tupleKey]
        )

    def getAllSubsetKeys(self, findKey):
        return [x for x in self.__rawSettings.keys() if self.isSubset(findKey, x)]

    def isSubset(self, tuple1, tuple2):
        set1 = set(tuple1)
        set2 = set(tuple2)
        if set2.issubset(set1):
            return True

        return False

    def generateConfig(self):
        childKeys = ArrUtils.sort_keys_by_len(self.getAllSubsetKeys(self.__reqKeyList))
        masterConfig = self.findSetting(["master"])
        for childkey in childKeys:
            masterConfig = DictUtils.merge(
                dict1=masterConfig,
                dict2=self.findSetting(childkey),
                merge_recursive=self.__mergeRecursive,
                arr_rm_dupes=self.__arr_rm_dupes,
            )
            masterConfig.pop("setting")

        if "setting" in masterConfig:
            masterConfig.pop("setting")

        return masterConfig

    def getConfig(self, key):
        keys = key.split(".")
        value = functools.reduce(
            lambda d, key: (d.get(key) if isinstance(d, dict) else None) if d else None,
            keys,
            self.__config,
        )
        if value is None:
            raise Exception("Config key:{} not found".format(key))
        return value

    def getConfigWithDefault(self, key, default):
        try:
            return self.getConfig(key)
        except Exception:
            return default



from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver
import json

from kfsd.apps.models.base import BaseModel
from kfsd.apps.core.utils.system import System
from kfsd.apps.core.utils.time import Time
from kfsd.apps.core.msmq.rabbitmq.base import RabbitMQ
from kfsd.apps.core.common.logger import Logger, LogLevel


logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class Outpost(BaseModel):
    STATUS_CHOICES = (
        ("P", "PENDING"),
        ("I", "IN-PROGRESS"),
        ("E", "ERROR"),
        ("C", "COMPLETED"),
    )

    msg_queue_info = models.JSONField(default=dict)
    msg = models.JSONField(default=dict)
    status = models.CharField(max_length=1, choices=STATUS_CHOICES, default="P")
    attempts = models.IntegerField(default=0)
    debug_info = models.JSONField(default=dict)

    def save(self, *args, **kwargs):
        if not self.identifier:
            self.identifier = System.uuid(32)
        return super().save(*args, **kwargs)

    class Meta:
        app_label = "models"
        verbose_name = "Outpost"
        verbose_name_plural = "Outpost"


@receiver(post_save, sender=Outpost)
def signal_send_msg(sender, instance, created, **kwargs):
    if created:
        send_msg(instance.id)


def send_msg(instance_id):
    instance = Outpost.objects.get(id=instance_id)
    exchangeName = instance.msg_queue_info["exchange_name"]
    queueName = instance.msg_queue_info["queue_name"]
    routingKey = instance.msg_queue_info["routing_key"]
    logger.info(
        "Sending msg for outpost id: {} to exchange: {}, queue: {}, routing key: {}".format(
            instance.identifier, exchangeName, queueName, routingKey
        )
    )
    try:
        rabbitMQ = RabbitMQ()
        rabbitMQ.publish_msg_and_close_connection(
            exchangeName,
            queueName,
            routingKey,
            json.dumps(instance.msg),
        )
        instance.delete()
    except Exception:
        oldDebugInfo = instance.debug_info
        currentTimeStr = Time.calculate_time(Time.current_time(), {"minutes": 0}, True)
        oldDebugInfo[currentTimeStr] = "Error occurred in publishing outpost msg"
        instance.debug_info = oldDebugInfo
        instance.attempts += 1
        instance.status = "E"
        instance.save()



from kfsd.apps.core.auth.base import BaseUser
from kfsd.apps.core.auth.api.token import TokenAuth
from kfsd.apps.core.utils.dict import DictUtils


class TokenUser(BaseUser, TokenAuth):
    def __init__(self, request):
        BaseUser.__init__(self)
        TokenAuth.__init__(self, request=request)
        tokenUserInfo = self.getTokenUserInfo()
        self.setUserInfo(tokenUserInfo)

    def getUserCookies(self):
        userInfo = self.getUserInfo()
        return DictUtils.get_by_path(userInfo, "data.cookies")

    def getEmail(self):
        return DictUtils.get_by_path(self.getUserInfo(), "data.user.email")

    def isEmailVerified(self):
        return DictUtils.get_by_path(self.getUserInfo(), "data.user.is_email_verified")

    def isStaff(self):
        return DictUtils.get_by_path(self.getUserInfo(), "data.user.is_staff")

    def isSuperuser(self):
        return DictUtils.get_by_path(self.getUserInfo(), "data.user.is_superuser")



from kfsd.apps.core.utils.dict import DictUtils


class BaseUser:
    is_active = False

    def __init__(self):
        self.__userInfo = {}

    def setUserInfo(self, userInfo):
        self.__userInfo = userInfo

    def getUserInfo(self):
        return self.__userInfo

    def isAuthenticated(self):
        return DictUtils.get(self.getUserInfo(), "status")

    def getIdentifier(self):
        return DictUtils.get_by_path(self.getUserInfo(), "data.user.identifier")

    def getAPIKey(self):
        return DictUtils.get_by_path(self.getUserInfo(), "data.user.api_key")

    def isActive(self):
        return DictUtils.get_by_path(self.getUserInfo(), "data.user.is_active")



"""
Provides YAML rendering support.
"""
from __future__ import unicode_literals
import json

from rest_framework.renderers import BaseRenderer
from kfsd.apps.core.common.yaml import Yaml
import yaml


class KubefacetsYAMLRenderer(BaseRenderer):
    """
    Renderer which serializes to YAML.
    """

    media_type = "application/yaml"
    format = "yaml"
    charset = "utf-8"

    def render(self, data, accepted_media_type=None, renderer_context=None):
        """
        Renders `data` into serialized YAML.
        """

        customYaml = Yaml()
        if data is None:
            return ""

        renderer_context = renderer_context or {}

        try:
            return customYaml.getYaml(customYaml.formattedValueByType(data))
        except Exception:
            jsonFormat = json.dumps(data)
            dictObj = json.loads(jsonFormat)
            return yaml.dump(dictObj)



from django.db import models
from django.core.validators import MinLengthValidator, MaxLengthValidator, RegexValidator
from kfsd.apps.models.constants import MAX_LENGTH, MIN_LENGTH, IDENTIFIER_REGEX_CONDITION


class BaseModelManager(models.Manager):
    def get_by_natural_key(self, identifier):
        return self.get(identifier=identifier)


class BaseModel(models.Model):
    class Meta:
        abstract = True

    identifier = models.CharField(
        unique=True,
        max_length=MAX_LENGTH,
        validators=[
            RegexValidator(
                regex=IDENTIFIER_REGEX_CONDITION,
                message="identifier doesnt match condition {}".format(IDENTIFIER_REGEX_CONDITION)
            ),
            MaxLengthValidator(MAX_LENGTH),
            MinLengthValidator(MIN_LENGTH)
        ]
    )
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    objects = BaseModelManager()

    def natural_key(self):
        return (self.identifier,)

    def __str__(self):
        return self.identifier



from kfsd.apps.endpoints.serializers.base import ErrorSerializer
from kfsd.apps.endpoints.views.docs.v1.utils import UtilsV1Doc
from kfsd.apps.endpoints.serializers.utils.arr import ArrUtilsInputReqSerializer, ArrUtilsOutputRespSerializer
from kfsd.apps.endpoints.serializers.utils.system import SystemInputReqSerializer, SystemOutputRespSerializer
from kfsd.apps.endpoints.serializers.utils.attr import AttrUtilsInputReqSerializer, AttrUtilsOutputRespSerializer


class UtilsDoc:
    @staticmethod
    def arr_view():
        return {
            "summary": "Array",
            "tags": ["UTILS"],
            "request": ArrUtilsInputReqSerializer,
            "responses": {
                200: ArrUtilsOutputRespSerializer,
                500: ErrorSerializer
            },
            "examples": UtilsV1Doc.arr_examples()
        }

    @staticmethod
    def system_view():
        return {
            "summary": "System",
            "tags": ["UTILS"],
            "request": SystemInputReqSerializer,
            "responses": {
                200: SystemOutputRespSerializer,
                500: ErrorSerializer
            },
            "examples": UtilsV1Doc.system_examples()
        }

    @staticmethod
    def attr_view():
        return {
            "summary": "Attr",
            "tags": ["UTILS"],
            "request": AttrUtilsInputReqSerializer,
            "responses": {
                200: AttrUtilsOutputRespSerializer,
                500: ErrorSerializer
            },
            "examples": UtilsV1Doc.attr_examples()
        }



from kfsd.apps.core.tests.base_api import BaseAPITestCases
from kfsd.apps.core.utils.dict import DictUtils


class DictUtilsTests(BaseAPITestCases):

    def test_dict_get_all_keys(self):
        data = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys.json')
        store_arr = []
        DictUtils.get_all_keys(store_arr, data)
        self.assertEqual(store_arr, ["bio", "first_name", "age", "address", "city", "type"])

    def test_dict_filter_by_keys_neg(self):
        data = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys.json')
        filteredDict = DictUtils.filter_by_keys_neg(data, ["bio"])
        self.assertEqual(filteredDict, {"type": "bio"})

    def test_dict_get(self):
        data = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys.json')
        typeVal = DictUtils.get(data, "type")
        self.assertEqual(typeVal, "bio")

    def test_dict_merge(self):
        dict1 = {"name": "Gokul", "bio": {"age": 41}}
        dict2 = {"type": "bio", "bio": {"address": {"city": "Bowmanville"}}}
        mergedDict = DictUtils.merge(dict1=dict1, dict2=dict2, merge_recursive=False)
        self.assertEqual(mergedDict, {"name": "Gokul", "type": "bio", "bio": {"address": {"city": "Bowmanville"}}})

        mergedDict = DictUtils.merge(dict1=dict1, dict2=dict2, merge_recursive=True)
        self.assertEqual(mergedDict, {"name": "Gokul", "type": "bio", "bio": {"age": 41, "address": {"city": "Bowmanville"}}})

    def test_dict_merge_arr(self):
        dict1 = {"arr1": [5, 1, 2, 3]}
        dict2 = {"arr1": [3, 4]}

        mergedDict1 = DictUtils.merge(dict1=dict1.copy(), dict2=dict2.copy(), merge_recursive=False)
        self.assertEqual(mergedDict1, {"arr1": [3, 4]})

        mergedDict2 = DictUtils.merge(dict1=dict1.copy(), dict2=dict2.copy(), merge_recursive=True, arr_rm_dupes=False)
        self.assertEqual(mergedDict2, {"arr1": [5, 1, 2, 3, 3, 4]})

        mergedDict3 = DictUtils.merge(dict1=dict1.copy(), dict2=dict2.copy(), merge_recursive=True, arr_rm_dupes=False, is_arr_sorted=False)
        self.assertEqual(mergedDict3, {"arr1": [5, 1, 2, 3, 3, 4]})

        mergedDict4 = DictUtils.merge(dict1=dict1.copy(), dict2=dict2.copy(), merge_recursive=True, arr_rm_dupes=False, is_arr_sorted=True)
        self.assertEqual(mergedDict4, {"arr1": [1, 2, 3, 3, 4, 5]})

        mergedDict5 = DictUtils.merge(dict1=dict1.copy(), dict2=dict2.copy(), merge_recursive=True, arr_rm_dupes=True, is_arr_sorted=True)
        self.assertEqual(mergedDict5, {"arr1": [1, 2, 3, 4, 5]})

    def test_dict_merge_arr_complex_norecursive(self):
        dict1 = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_merge_arr_complex_dict1.json')
        dict2 = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_merge_arr_complex_dict2.json')

        obsDict1 = DictUtils.merge(dict1=dict1.copy(), dict2=dict2.copy(), merge_recursive=False, is_arr_sorted=True)
        expDict1 = self.readJSONData('kfsd/apps/core/tests/v1/data/responses/utils/dict/test_dict_merge_arr_complex_norecursive.json')
        self.assertEqual(obsDict1, expDict1)

    def test_dict_merge_arr_complex_recursive_no_dupes_no_arr_sorting(self):
        dict1 = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_merge_arr_complex_dict1.json')
        dict2 = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_merge_arr_complex_dict2.json')

        obsDict2 = DictUtils.merge(dict1=dict1.copy(), dict2=dict2.copy(), merge_recursive=True, arr_rm_dupes=False, is_arr_sorted=False)
        expDict2 = self.readJSONData('kfsd/apps/core/tests/v1/data/responses/utils/dict/test_dict_merge_arr_complex_recursive_no_dupes_no_arr_sorting.json')
        self.assertEqual(obsDict2, expDict2)

    def test_dict_merge_arr_complex_recursive_dupes_arr_sorting(self):
        dict1 = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_merge_arr_complex_dict1.json')
        dict2 = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_merge_arr_complex_dict2.json')

        obsDict3 = DictUtils.merge(dict1=dict1.copy(), dict2=dict2.copy(), merge_recursive=True, arr_rm_dupes=True, is_arr_sorted=True)
        expDict3 = self.readJSONData('kfsd/apps/core/tests/v1/data/responses/utils/dict/test_dict_merge_arr_complex_recursive_dupes_arr_sorting.json')
        self.assertEqual(obsDict3, expDict3)

    def test_dict_create_hierarchy(self):
        newDict = DictUtils.create_hierarchy(["bio", "address", "city"], "Bowmanville")
        self.assertEqual(
            newDict,
            {
                "bio": {
                    "address": {
                        "city": "Bowmanville"
                    }
                }
            }
        )

    def test_key_exists(self):
        dict1 = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_merge_arr_complex_dict1.json')
        self.assertTrue(DictUtils.key_exists(dict1, "templates"))
        self.assertFalse(DictUtils.key_exists(dict1, "bio"))

    def test_get_by_path(self):
        dict1 = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys.json')
        val = DictUtils.get_by_path(dict1, "bio.address.city")
        self.assertEqual(val, "Bowmanville")

    def test_filter_by_keys(self):
        dict1 = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys.json')
        val = DictUtils.filter_by_keys(dict1, ["bio", "type"])
        self.assertEqual(dict1, val)

        val = DictUtils.filter_by_keys(dict1, ["type"])
        self.assertEqual(val, {"type": "bio"})

    def test_key_exists_multi(self):
        dict1 = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/utils/dict/test_dict_get_all_keys.json')
        val = DictUtils.key_exists_multi(dict1, ["bio", "type"])
        self.assertTrue(val)

        val1 = DictUtils.key_exists_multi(dict1, ["bio", "type", "name"])
        self.assertFalse(val1)

    def test_create(self):
        dict1 = DictUtils.create(name="gokul", age=41, location="Bangalore")
        self.assertEqual(dict1, {"name": "gokul", "age": 41, "location": "Bangalore"})

    def test_sort_by_values(self):
        dict1 = {
            "key1": "abc",
            "key2": "bac",
            "key3": "cba",
        }
        val = DictUtils.sort_by_values(dict1, False)
        self.assertEqual(val, dict1)

        val1 = DictUtils.sort_by_values(dict1, True)
        self.assertEqual(val1, {"key3": "cba", "key2": "bac", "key1": "abc"})



from rest_framework import serializers
from kfsd.apps.endpoints.serializers.base import BaseInputReqSerializer, BaseOutputRespSerializer


class ConfigurationInputReqSerializer(BaseInputReqSerializer):
    op = serializers.ChoiceField(choices=["CONFIG"])


class ConfigurationOutputRespSerializer(BaseOutputRespSerializer):
    op = serializers.ChoiceField(choices=["CONFIG"])



from rest_framework import serializers
from kfsd.apps.endpoints.serializers.base import (
    BaseInputReqSerializer,
    BaseOutputRespSerializer,
)


class ArrUtilsInputReqSerializer(BaseInputReqSerializer):
    op = serializers.ChoiceField(choices=["JOIN", "INTERSECTION", "MERGE"])


class ArrUtilsOutputRespSerializer(BaseOutputRespSerializer):
    op = serializers.ChoiceField(choices=["JOIN", "INTERSECTION", "MERGE"])



from kfsd.apps.core.common.logger import Logger, LogLevel

from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.utils.attr import AttrUtils
from kfsd.apps.endpoints.serializers.utils.attr import AttrUtilsOutputRespSerializer


class AttrHandler:
    OP = "op"
    EXPR = "EXPR"

    def __init__(self, input):
        self.__logger = Logger.getSingleton(__name__, LogLevel.DEBUG)
        self.__input = input

    def methodMappings(self, op):
        mapping = {
            self.EXPR: self.getAttr
        }
        return mapping[op]

    def gen(self):
        op = DictUtils.get(self.__input, self.OP)
        return self.methodMappings(op)()

    def genOutput(self, key, value):
        data = {
            "op": key,
            "output": {
                "value": value
            }
        }
        outputSerializer = AttrUtilsOutputRespSerializer(data=data)
        outputSerializer.is_valid()
        return outputSerializer.data

    def evaluate(self, expr, request, default=None):
        result = default
        try:
            result = eval(expr)
        except Exception as e:
            self.__logger.error("{} not found".format(e.__str__()))
        return result

    def getAttr(self):
        dictVal = DictUtils.get_by_path(self.__input, "input.dict")
        expr = DictUtils.get_by_path(self.__input, "input.expr")
        request = AttrUtils.format(dictVal)
        return self.genOutput(self.EXPR, self.evaluate(expr, request))



from setuptools import setup, find_packages

setup(
    name="kfsd",
    version="0.0.57",
    description="Sample App",
    long_description="Sample Pkg",
    long_description_content_type="text/markdown",
    author="Gokul Nathan",
    author_email="nathangokul111@gmail.com",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "Django==4.2.1",
        "asgiref==3.6.0",
        "coverage==7.1.0",
        "djangorestframework==3.14.0",
        "flake8==6.0.0",
        "mccabe==0.7.0",
        "netifaces==0.11.0",
        "pycodestyle==2.10.0",
        "pyflakes==3.0.1",
        "pytz==2022.7.1",
        "PyYAML==6.0",
        "sqlparse==0.4.3",
        "drf-spectacular==0.26.2",
        "ruamel.yaml==0.17.21",
        "requests==2.28.2",
        "pika==1.3.2",
        "SQLAlchemy==2.0.16",
    ],
)



from django.core.cache import cache as DjangoCache
from functools import wraps


def cache(key, timeout=3600):
    def get(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            data = DjangoCache.get(key)
            if data is None:
                data = func(*args, **kwargs)
                DjangoCache.set(key, data, timeout)
            return data

        return wrapper

    return get



from rest_framework import generics, status
from rest_framework.views import Response

from kfsd.apps.endpoints.renderers.kubefacetsjson import KubefacetsJSONRenderer
from kfsd.apps.endpoints.views.pagination.model import ModelPagination


class SerializerEvalView(generics.GenericAPIView):
    renderer_classes = [KubefacetsJSONRenderer]
    pagination_class = ModelPagination

    def processRequest(self, request):
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            validated_data = serializer.validated_data
            validated_data["request"] = request
            return serializer.eval(validated_data)
        else:
            return Response(serializer.errors, status.HTTP_400_BAD_REQUEST)

    def post(self, request, *args, **kwargs):
        return self.processRequest(request)



from kfsd.apps.core.common.kubefacets_config import KubefacetsConfig


class KubefacetsConfigMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
        self.__config = self.genConfig()

    def genConfig(self):
        return KubefacetsConfig().getConfig()

    def __call__(self, request):
        request.config = self.__config
        response = self.get_response(request)
        return response

    def process_view(self, request, view_func, view_args, view_kwargs):
        return None



import urllib.parse
import requests
import urllib3.exceptions
from rest_framework import status

from kfsd.apps.core.exceptions.api import KubefacetsAPIException
from kfsd.apps.core.utils.http.response import Response


class HTTP(Response):
    def formatUrl(self, args):
        if None in args:
            raise KubefacetsAPIException(
                "Invalid url paths passed for forming url",
                "url_error",
                status.HTTP_400_BAD_REQUEST,
                "Probably config error, plz check if services are up?",
            )
        return "/".join(args)

    def genUrlEncode(self, urlstr):
        return urllib.parse.quote(urlstr)

    def request(self, method, url, expStatus, **kwargs):
        try:
            resp = method(url, **kwargs)
            self.setResponse(resp)
            if self.isRespValid(expStatus):
                return resp
        except requests.exceptions.Timeout as e:
            raise KubefacetsAPIException(
                "The server took too long to respond to your request. Please try again later.",
                "server_timed_out",
                status.HTTP_408_REQUEST_TIMEOUT,
                "url: {}, error: {}".format(url, str(e)),
            )
        except requests.exceptions.ConnectionError as e:
            raise KubefacetsAPIException(
                "Service temporarily unavailable. Please try again later.",
                "service_unavailable",
                status.HTTP_503_SERVICE_UNAVAILABLE,
                "url: {}, error: {}".format(url, str(e)),
            )
        except urllib3.exceptions.NewConnectionError as e:
            raise KubefacetsAPIException(
                "Service temporarily unavailable. Please try again later.",
                "service_unavailable",
                status.HTTP_503_SERVICE_UNAVAILABLE,
                "url: {}, error: {}".format(url, str(e)),
            )

    def post(self, url, expStatus, **kwargs):
        return self.request(requests.post, url, expStatus, **kwargs)

    def get(self, url, expStatus, **kwargs):
        return self.request(requests.get, url, expStatus, **kwargs)

    def delete(self, url, expStatus, **kwargs):
        return self.request(requests.delete, url, expStatus, **kwargs)



from rest_framework import serializers
from kfsd.apps.endpoints.serializers.base import BaseInputReqSerializer, BaseOutputRespSerializer


class AttrUtilsInputReqSerializer(BaseInputReqSerializer):
    op = serializers.ChoiceField(choices=["EXPR"])


class AttrUtilsOutputRespSerializer(BaseOutputRespSerializer):
    op = serializers.ChoiceField(choices=["EXPR"])



from kfsd.apps.core.tests.base_api import BaseAPITestCases
from kfsd.apps.core.common.configuration import Configuration


class ConfigurationTests(BaseAPITestCases):

    def test_configuration_dev(self):
        data = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/common/configuration/gateway.json')
        dimensions = {"environment": "k8s", "cluster": "mac", "type": "dev"}
        configObj = Configuration(settings=data, dimensions=dimensions, merge_recursive=True, arr_rm_dupes=True)
        obsConfig = configObj.getFinalConfig()
        expConfig = self.readJSONData('kfsd/apps/core/tests/v1/data/responses/common/configuration/test_configuration_dev.json')
        self.assertEqual(obsConfig, expConfig)

    def test_configuration_prod(self):
        data = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/common/configuration/gateway.json')
        dimensions = {"environment": "k8s", "cluster": "inhouse", "type": "prod"}
        configObj = Configuration(settings=data, dimensions=dimensions, merge_recursive=True, arr_rm_dupes=True)
        obsConfig = configObj.getFinalConfig()
        expConfig = self.readJSONData('kfsd/apps/core/tests/v1/data/responses/common/configuration/test_configuration_prod.json')
        self.assertEqual(obsConfig, expConfig)

    def test_configuration_nomerge_dev(self):
        data = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/common/configuration/gateway.json')
        dimensions = {"environment": "k8s", "cluster": "mac", "type": "dev"}
        configObj = Configuration(settings=data, dimensions=dimensions, merge_recursive=False, arr_rm_dupes=False)
        obsConfig = configObj.getFinalConfig()
        expConfig = self.readJSONData('kfsd/apps/core/tests/v1/data/responses/common/configuration/test_configuration_nomerge_dev.json')
        self.assertEqual(obsConfig, expConfig)

    def test_configuration_nomerge_prod(self):
        data = self.readJSONData('kfsd/apps/core/tests/v1/data/requests/common/configuration/gateway.json')
        dimensions = {"environment": "k8s", "cluster": "inhouse", "type": "prod"}
        configObj = Configuration(settings=data, dimensions=dimensions, merge_recursive=False, arr_rm_dupes=False)
        obsConfig = configObj.getFinalConfig()
        expConfig = self.readJSONData('kfsd/apps/core/tests/v1/data/responses/common/configuration/test_configuration_nomerge_prod.json')
        self.assertEqual(obsConfig, expConfig)



from kfsd.apps.core.common.logger import Logger, LogLevel
import ruamel.yaml
from ruamel.yaml import YAML
from ruamel.yaml.compat import StringIO
from ruamel.yaml.scalarstring import LiteralScalarString


class Yaml(YAML):
    def __init__(self):
        self.__logger = Logger.getSingleton(__name__, LogLevel.DEBUG)
        self.__YAML = YAML()

    def getRuamelYaml(self):
        return self.__YAML

    def isMultiLineKey(self, key):
        return key.endswith("|")

    def formatMultiLineKey(self, key):
        return key[:-1]

    def formattedKey(self, key):
        return (self.formatMultiLineKey(key) if self.isMultiLineKey(key) else key)

    def isMultiLineStrVal(self, value):
        return True if ("\n" in value) else False

    def convertValToYamlByType(self, value):
        if isinstance(value, str) and self.isMultiLineStrVal(value):
            return self.convertStrToMultiLine(value)
        elif isinstance(value, dict):
            return self.convertDictToMultiLine(value)
        elif isinstance(value, list):
            return self.convertListToMultiLine(value)

        return value

    def convertStrToMultiLine(self, value):
        return LiteralScalarString(self.formatFinalMultiLineStr("""\
{}
""".format(value)))

    def convertDictToMultiLine(self, value):
        dictYamlStr = self.getYaml(value)
        return LiteralScalarString(self.formatFinalMultiLineStr("""\
{}
""".format(dictYamlStr)))

    def convertListToMultiLine(self, value):
        return LiteralScalarString(self.formatFinalMultiLineStr("""\
{}
""".format("\n".join(value[0:]))))

    def formatFinalMultiLineStr(self, str):
        return str.strip()+"\n"

    def formatListConfig(self, value):
        return [self.formattedValueByType(item) for item in value]

    def formattedDictValue(self, key, value):
        if self.isMultiLineKey(key):
            return self.convertValToYamlByType(value)

        return self.formattedValueByType(value)

    def formattedValueByType(self, value):
        if isinstance(value, dict):
            return self.formatDictConfig(value)
        elif isinstance(value, list):
            return self.formatListConfig(value)
        elif isinstance(value, str) and self.isMultiLineStrVal(value):
            return self.convertStrToMultiLine(value)

        return value

    def formatDictConfig(self, config):
        return {self.formattedKey(k): self.formattedDictValue(k, v) for (k, v) in config.items()}

    def getYaml(self, data, **kw):
        stream = StringIO()
        self.__YAML.dump(data, stream, **kw)
        return stream.getvalue()

    def getPythonObj(self, config):
        return ruamel.yaml.safe_load(self.getYaml(config))



import requests
import json

from kfsd.apps.core.utils.http.base import HTTP
from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.core.utils.http.django.request import DjangoRequest
from kfsd.apps.core.utils.http.headers.base import Headers
from kfsd.apps.core.utils.http.headers.contenttype import ContentType
from kfsd.apps.core.exceptions.api import KubefacetsAPIException
from kfsd.apps.core.common.logger import Logger, LogLevel

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


class APIGateway(Headers, HTTP):
    def __init__(self, request=None):
        self.__request = DjangoRequest(request)
        Headers.__init__(self)
        HTTP.__init__(self)

    def getServicesAPIKey(self):
        return self.getDjangoRequest().findConfigs(["services.api_key"])[0]

    def getDjangoRequest(self):
        return self.__request

    def setErrorResponse(self, e):
        response = requests.models.Response()
        response.status_code = 200
        response.headers = {"Content-Type": "application/json"}
        errorResp = self.constructGatewayResp(False, {}, self.readExceptionError(e))
        response._content = bytes(json.dumps(errorResp), "utf-8")
        self.setResponse(response)
        logger.logWebRequestError(
            self.getDjangoRequest().getRequest(), self.readExceptionError(e), "error"
        )

    def setSuccessResponse(self, resp):
        response = requests.models.Response()
        response.status_code = 200
        response.headers = {"Content-Type": "application/json"}
        successResp = self.constructGatewayResp(True, resp, {})
        response._content = bytes(json.dumps(successResp), "utf-8")
        self.setResponse(response)

    def isHTTPRequestSuccessfull(self):
        return DictUtils.get(self.getJsonResponse(), "status")

    def constructGatewayResp(self, status, data, errors={}):
        return {
            "status": status,
            "data": data,
            "error": errors,
        }

    def readExceptionError(self, e):
        return {
            "detail": e.detail,
            "status_code": e.status_code,
            "default_code": e.default_code,
            "type": "error",
        }

    def setHttpHeaders(self):
        self.setAPIKey(self.getServicesAPIKey())
        self.setContentType(ContentType.APPLICATION_JSON)

    def isGatewayFormatResp(self):
        return DictUtils.key_exists_multi(self.getJsonResponse(), ["status", "data", "error"])

    def httpPost(self, postUrl, payload, expStatus):
        try:
            self.setHttpHeaders()
            self.post(postUrl, expStatus, json=payload, headers=self.getReqHeaders())
            if not self.isGatewayFormatResp():
                self.setSuccessResponse(self.getJsonResponse())
                return self.getJsonResponse()
            return self.getJsonResponse()
        except KubefacetsAPIException as e:
            self.setErrorResponse(e)
            return self.getJsonResponse()

    def httpGet(self, getUrl, expStatus):
        try:
            self.setHttpHeaders()
            self.get(getUrl, expStatus, headers=self.getReqHeaders())
            if not self.isGatewayFormatResp():
                self.setSuccessResponse(self.getJsonResponse())
                return self.getJsonResponse()
            return self.getJsonResponse()
        except KubefacetsAPIException as e:
            self.setErrorResponse(e)
            return self.getJsonResponse()

    def httpDel(self, getUrl, expStatus):
        try:
            self.setHttpHeaders()
            self.delete(getUrl, expStatus, headers=self.getReqHeaders())
            if not self.isGatewayFormatResp():
                self.setSuccessResponse({})
                return self.getJsonResponse()
            return self.getJsonResponse()
        except KubefacetsAPIException as e:
            self.setErrorResponse(e)
            return self.getJsonResponse()

    def constructUrl(self, configPaths):
        uris = self.getDjangoRequest().findConfigs(configPaths)
        return self.formatUrl(uris)




class KubefacetsFEException(Exception):
    msg = "Unexpected Error"
    status_code = 500
    redirect_url = "/"

    def __init__(self, msg, code, redirectUrl):
        self.msg = msg
        self.status_code = code
        self.redirect_url = redirectUrl
